"""
This class should be used to create a window widget for the LLM.
"""
from PySide6.QtCore import Slot
from PySide6.QtWidgets import QWidget

from airunner.enums import SignalCode
from airunner.settings import DEFAULT_LLM_HF_PATH
from airunner.widgets.base_widget import BaseWidget
from airunner.widgets.llm.templates.llm_settings_ui import Ui_llm_settings_widget
from airunner.windows.main.ai_model_mixin import AIModelMixin


class LLMSettingsWidget(
    BaseWidget,
    AIModelMixin
):
    widget_class_ = Ui_llm_settings_widget

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        AIModelMixin.__init__(self)
        self.ui.model_type_container.hide()
        self.ui.model_version_container.hide()
        self.ui.prompt_template_container.hide()

    @Slot(bool)
    def toggle_use_cache(self, val: bool):
        self.update_chatbot("use_cache", val)

    def showEvent(self, event):
        super().showEvent(event)
        self.emit_signal(SignalCode.WINDOW_LOADED_SIGNAL)

    def early_stopping_toggled(self, val):
        self.update_chatbot("early_stopping", val)

    def do_sample_toggled(self, val):
        self.update_chatbot("do_sample", val)
    
    def toggle_leave_model_in_vram(self, val):
        if val:
            self.update_memory_settings("unload_unused_models", False)
            self.update_memory_settings("move_unused_model_to_cpu", False)

    def initialize_form(self):
        elements = [
            self.ui.prompt_template,
            self.ui.model,
            self.ui.model_version,
            self.ui.random_seed,
            self.ui.do_sample,
            self.ui.early_stopping,
            self.ui.override_parameters,
            self.ui.use_cache,
        ]

        for element in elements:
            element.blockSignals(True)

        self.ui.top_p.init(slider_callback=self.callback, current_value=self.llm_generator_settings.top_p)
        self.ui.max_new_tokens.init(slider_callback=self.callback, current_value=self.llm_generator_settings.max_new_tokens)
        self.ui.repetition_penalty.init(slider_callback=self.callback, current_value=self.llm_generator_settings.repetition_penalty)
        self.ui.min_length.init(slider_callback=self.callback, current_value=self.llm_generator_settings.min_length)
        self.ui.length_penalty.init(slider_callback=self.callback, current_value=self.llm_generator_settings.length_penalty)
        self.ui.num_beams.init(slider_callback=self.callback, current_value=self.llm_generator_settings.num_beams)
        self.ui.ngram_size.init(slider_callback=self.callback, current_value=self.llm_generator_settings.ngram_size)
        self.ui.temperature.init(slider_callback=self.callback, current_value=self.llm_generator_settings.temperature)
        self.ui.sequences.init(slider_callback=self.callback, current_value=self.llm_generator_settings.sequences)
        self.ui.top_k.init(slider_callback=self.callback, current_value=self.llm_generator_settings.top_k)

        self.ui.override_parameters.setChecked(self.llm_generator_settings.override_parameters)

        self.ui.use_cache.setChecked(self.chatbot.use_cache)

        # get unique model names
        self.ui.model.clear()
        self.ui.model.addItems([
            "seq2seq",
            "causallm",
            "visualqa",
        ])
        self.ui.model.setCurrentText(self.application_settings.current_llm_generator)

        templates = {
            "Mistral 7B Instruct: Default Chatbot": dict(
                name="Mistral 7B Instruct: Default Chatbot",
                model=DEFAULT_LLM_HF_PATH,
                llm_category="causallm",
            ),
        }
        names = [v["name"] for k, v in templates.items()]

        self.ui.prompt_template.blockSignals(True)
        self.ui.prompt_template.clear()
        self.ui.prompt_template.addItems(names)
        template_name = self.llm_generator_settings.prompt_template
        if template_name == "":
            template_name = names[0]
            self.update_llm_generator_settings("prompt_template", template_name)
        self.ui.prompt_template.setCurrentText(template_name)
        self.ui.prompt_template.blockSignals(False)

        self.update_model_version_combobox()
        self.ui.model_version.setCurrentText(
            self.chatbot.model_version
        )
        self.ui.random_seed.setChecked(
            self.chatbot.random_seed
        )
        self.ui.do_sample.setChecked(
            self.chatbot.do_sample
        )
        self.ui.early_stopping.setChecked(
            self.chatbot.early_stopping
        )

        for element in elements:
            element.blockSignals(False)

    def callback(self, attr_name, value, widget=None):
        self.update_chatbot(attr_name, value)

    def model_text_changed(self, val):
        self.update_application_settings("current_llm_generator", val)
        self.update_model_version_combobox()
        self.model_version_changed(self.ui.model_version.currentText())
        self.initialize_form()

    def model_version_changed(self, val):
        self.update_llm_generator_settings("model_version", val)
    
    def toggle_move_model_to_cpu(self, val):
        self.update_memory_settings("move_unused_model_to_cpu", val)
        if val:
            self.update_memory_settings("unload_unused_models", False)

    def override_parameters_toggled(self, val):
        self.update_llm_generator_settings("override_parameters", val)
        
    def prompt_template_text_changed(self, value):
        self.update_llm_generator_settings("prompt_template", value)
        
    def random_seed_toggled(self, val):
        self.update_chatbot("random_seed", val)
        
    def seed_changed(self, val):
        self.update_chatbot("seed", val)
        
    def toggle_unload_model(self, val):
        self.update_memory_settings("unload_unused_models", val)
        if val:
            self.update_memory_settings("move_unused_model_to_cpu", False)

    def reset_settings_to_default_clicked(self):
        self.initialize_form()
        chatbot = self.chatbot
        self.ui.top_p.set_slider_and_spinbox_values(chatbot.top_p)
        self.ui.repetition_penalty.set_slider_and_spinbox_values(chatbot.repetition_penalty)
        self.ui.min_length.set_slider_and_spinbox_values(chatbot.min_length)
        self.ui.length_penalty.set_slider_and_spinbox_values(chatbot.length_penalty)
        self.ui.num_beams.set_slider_and_spinbox_values(chatbot.num_beams)
        self.ui.ngram_size.set_slider_and_spinbox_values(chatbot.ngram_size)
        self.ui.temperature.set_slider_and_spinbox_values(chatbot.temperature)
        self.ui.sequences.set_slider_and_spinbox_values(chatbot.sequences)
        self.ui.top_k.set_slider_and_spinbox_values(chatbot.top_k)
        self.ui.random_seed.setChecked(chatbot.random_seed)

    def update_model_version_combobox(self):
        self.ui.model_version.blockSignals(True)
        self.ui.model_version.clear()
        ai_model_paths = self.ai_model_paths(model_type="llm", pipeline_action=self.ui.model.currentText())
        self.ui.model_version.addItems(ai_model_paths)
        self.ui.model_version.blockSignals(False)

    def set_tab(self, tab_name):
        index = self.ui.tabWidget.indexOf(self.ui.tabWidget.findChild(QWidget, tab_name))
        self.ui.tabWidget.setCurrentIndex(index)

    def update_chatbot(self, key, val):
        chatbot = self.chatbot
        try:
            setattr(chatbot, key, val)
        except TypeError:
            self.logger.error(f"Attribute {key} does not exist in Chatbot")
            return
        self.save_object(chatbot)



import threading

from airunner.handlers.stt.whisper_handler import WhisperHandler
from airunner.enums import SignalCode
from airunner.workers.worker import Worker


class AudioProcessorWorker(Worker):
    """
    This class is responsible for processing audio.
    It will process audio from the audio_queue and send it to the model.
    """ 
    fs = 0

    def __init__(self):
        self._stt = None
        super().__init__(signals = (
            (SignalCode.APPLICATION_SETTINGS_CHANGED_SIGNAL, self.update_properties),
            (SignalCode.STT_LOAD_SIGNAL, self.on_stt_load_signal),
            (SignalCode.STT_UNLOAD_SIGNAL, self.on_stt_unload_signal),
            (SignalCode.AUDIO_CAPTURE_WORKER_RESPONSE_SIGNAL, self.on_stt_process_audio_signal),
        ))

    def start_worker_thread(self):
        self._initialize_stt_handler()
        if self.application_settings.stt_enabled:
            self._stt.load()

    def _initialize_stt_handler(self):
        if self._stt is None:
            self._stt = WhisperHandler()

    def on_stt_load_signal(self):
        if self._stt:
            threading.Thread(target=self._stt_load).start()

    def on_stt_unload_signal(self):
        if self._stt:
            threading.Thread(target=self._stt_unload).start()

    def unload(self):
        self._stt_unload()

    def load(self):
        self._initialize_stt_handler()
        self._stt_load()

    def _stt_load(self):
        if self._stt:
            self._stt.load()
            self.emit_signal(SignalCode.STT_START_CAPTURE_SIGNAL)

    def _stt_unload(self):
        self.emit_signal(SignalCode.STT_STOP_CAPTURE_SIGNAL)
        if self._stt:
            self._stt.unload()

    def on_stt_process_audio_signal(self, message):
        self.add_to_queue(message)

    def handle_message(self, audio_data):
        self._stt.process_audio(audio_data)
    
    def update_properties(self):
        self.fs = self.stt_settings.fs



from PySide6.QtWidgets import QWizardPage, QVBoxLayout, QWizard
from airunner.mediator_mixin import MediatorMixin
from airunner.windows.main.settings_mixin import SettingsMixin


class BaseWizard(
    QWizardPage,
    MediatorMixin,
    SettingsMixin
):
    class_name_ = None
    widget_class_ = None

    def __init__(self, parent: QWizard):
        MediatorMixin.__init__(self)
        super(BaseWizard, self).__init__()
        
        if self.class_name_:
            self.ui = self.class_name_()
            self.ui.setupUi(self)

        if self.widget_class_:
            widget = self.widget_class_()
            layout = QVBoxLayout()
            layout.addWidget(widget)
            self.setLayout(layout)

        self.parent = parent

    def initialize_form(self):
        """
        Override this function to initialize form based on specific page in question.
        :return:
        """
        pass



"""Initial migration

Revision ID: 72d9134823cb
Revises: 
Create Date: 2024-10-07 17:18:54.617216

"""
from typing import Sequence, Union

import sqlalchemy
from alembic import op
import sqlalchemy as sa

from airunner.data.models import settings_models
from airunner.data.models.settings_models import Base
from airunner.data.bootstrap.controlnet_bootstrap_data import controlnet_bootstrap_data
from airunner.data.bootstrap.font_settings_bootstrap_data import font_settings_bootstrap_data
from airunner.data.bootstrap.imagefilter_bootstrap_data import imagefilter_bootstrap_data
from airunner.data.bootstrap.model_bootstrap_data import model_bootstrap_data
from airunner.data.bootstrap.pipeline_bootstrap_data import pipeline_bootstrap_data
from airunner.data.bootstrap.prompt_templates_bootstrap_data import prompt_templates_bootstrap_data
from airunner.settings import SCHEDULER_CLASSES, DEFAULT_SHORTCUTS

# revision identifiers, used by Alembic.
revision: str = '72d9134823cb'
down_revision: Union[str, None] = None
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # Get all model classes
    models = Base.__subclasses__()

    # Iterate over each model
    for model in models:
        op.create_table(
            model.__tablename__,
            *model.__table__.columns
        )
        if not model.__tablename__ in (
            "shortcut_keys",
            "lora",
            "embeddings",
            "prompt_templates",
            "aimodels",
            "schedulers",
            "controlnet_models",
            "font_settings",
            "pipeline_models",
            "messages",
            "summaries",
            "image_filter_settings",
            "image_filter_values",
            "conversations",
            "saved_prompts",
            "target_directories",
            "target_files",
        ):
            try:
                set_default_values(model)
            except Exception as e:
                print(e)

    set_default_ai_models()
    set_default_schedulers()
    set_default_shortcut_keys()
    set_default_prompt_templates()
    set_default_controlnet_models()
    set_default_font_settings()
    set_default_pipeline_values()
    set_image_filter_settings()
    # ### end Alembic commands ###


def set_default_values(model_name_):
    default_values = {}
    for column in model_name_.__table__.columns:
        if column.default is not None:
            default_values[column.name] = column.default.arg
    op.bulk_insert(
        model_name_.__table__,
        [default_values]
    )

def set_default_ai_models():
    values = []
    for model in model_bootstrap_data:
        values.append(model)
    op.bulk_insert(
        settings_models.AIModels.__table__,
        values
    )

def set_default_schedulers():
    op.bulk_insert(
        settings_models.Schedulers.__table__,
        SCHEDULER_CLASSES
    )

def set_default_shortcut_keys():
    op.bulk_insert(
        settings_models.ShortcutKeys.__table__,
        DEFAULT_SHORTCUTS
    )

def set_default_prompt_templates():
    values = []
    for template in prompt_templates_bootstrap_data:
        values.append(template)
    op.bulk_insert(
        settings_models.PromptTemplate.__table__,
        values
    )

def set_default_controlnet_models():
    values = []
    for template in controlnet_bootstrap_data:
        values.append(template)
    op.bulk_insert(
        settings_models.ControlnetModel.__table__,
        values
    )

def set_default_font_settings():
    for font_setting in font_settings_bootstrap_data:
        op.bulk_insert(
            settings_models.FontSetting.__table__,
            [font_setting]
        )

def set_default_pipeline_values():
    for pipeline in pipeline_bootstrap_data:
        op.bulk_insert(
            settings_models.PipelineModel.__table__,
            [pipeline]
        )

def set_image_filter_settings():
    connection = op.get_bind()
    for filter_name, filter_data in imagefilter_bootstrap_data.items():
        result = connection.execute(
            sa.text(
                "INSERT INTO image_filter_settings (name, display_name, auto_apply, filter_class) "
                "VALUES (:name, :display_name, :auto_apply, :filter_class)"
            ),
            {
                'name': filter_data['name'],
                'display_name': filter_data['display_name'],
                'auto_apply': filter_data['auto_apply'],
                'filter_class': filter_data['filter_class']
            }
        )
        filter_id = result.lastrowid
        for value_name, value_data in filter_data['image_filter_values'].items():
            connection.execute(
                sa.text(
                    "INSERT INTO image_filter_values (name, value, value_type, min_value, max_value, image_filter_id) "
                    "VALUES (:name, :value, :value_type, :min_value, :max_value, :image_filter_id)"
                ),
                {
                    'name': value_data['name'],
                    'value': value_data['value'],
                    'value_type': value_data['value_type'],
                    'min_value': value_data['min_value'],
                    'max_value': value_data['max_value'],
                    'image_filter_id': filter_id
                }
            )

def downgrade() -> None:
    models = Base.__subclasses__()
    for model in models:
        try:
            op.drop_table(model.__tablename__)
        except sqlalchemy.exc.OperationalError:
            pass
        set_default_values(model)
    # ### end Alembic commands ###



import os
import re
import time
from queue import Queue

import inflect
import torch
from transformers import AutoTokenizer

from airunner.handlers.tts.tts_handler import TTSHandler
from airunner.enums import SignalCode, LLMChatRole, ModelType, ModelStatus
from airunner.utils.clear_memory import clear_memory


class SpeechT5TTSHandler(TTSHandler):
    target_model = "t5"

    def __init__(self, *args, **kwargs):
        from transformers import SpeechT5Processor, SpeechT5ForTextToSpeech
        self._model_class_ = SpeechT5ForTextToSpeech
        self._processor_class_ = SpeechT5Processor
        self._current_model = None
        self._character_replacement_map = {
            "\n": " ",
            "’": "'",
            "-": " "
        }
        self._single_character_sentence_enders = [".", "?", "!", "…"]
        self._double_character_sentence_enders = [".”", "?”", "!”", "…”", ".'", "?'", "!'", "…'"]
        self._model = None
        self._vocoder = None
        self._processor = None
        self._text_queue = Queue()
        self._input_text = ""
        self._corpus = []
        self._speaker_embeddings = None
        self._dataset = None
        self._sentences = []
        self._do_interrupt = False
        self._cancel_generated_speech = False
        self._paused = False
        super().__init__(*args, **kwargs)

    @property
    def processor_path(self) -> str:
        path:str = self.speech_t5_settings.processor_path
        return os.path.expanduser(
            os.path.join(
                self.path_settings.base_path,
                "text/models",
                "tts",
                path
            )
        )

    @property
    def model_path(self) -> str:
        path:str = self.speech_t5_settings.model_path
        return os.path.expanduser(
            os.path.join(
                self.path_settings.base_path,
                "text/models",
                "tts",
                path
            )
        )

    @property
    def vocoder_path(self) -> str:
        path:str = self.speech_t5_settings.vocoder_path
        return os.path.expanduser(
            os.path.join(
                self.path_settings.base_path,
                "text/models",
                "tts",
                path
            )
        )

    @property
    def speaker_embeddings_path(self):
        return os.path.expanduser(
            os.path.join(
                self.path_settings.base_path,
                "text/models",
                "tts",
                "datasets",
                "w4ffl35",
                "speecht5_speaker_embeddings",
                "speaker_embeddings"
            )
        )

    def generate(self, message):
        if self.model_status is not ModelStatus.LOADED:
            return None

        if self._do_interrupt or self._paused:
            return None
        try:
            return self._do_generate(message)
        except torch.cuda.OutOfMemoryError:
            self.logger.error("Out of memory")
            return None

    def load(self, target_model=None):
        if self.model_status is ModelStatus.LOADING:
            return
        if self.model_status in (
            ModelStatus.LOADED,
            ModelStatus.READY,
            ModelStatus.FAILED
        ):
            self.unload()
        self.change_model_status(ModelType.TTS, ModelStatus.LOADING)
        self.logger.debug(f"Loading text-to-speech")
        self._load_model()
        self._load_vocoder()
        self._load_processor()
        self._load_speaker_embeddings()
        self._load_tokenizer()
        self._load_corpus()
        self._current_model = self._current_model

        if (
            self._model is not None
            and self._vocoder is not None
            and self._processor is not None
            and self._speaker_embeddings is not None
            and self._tokenizer is not None
            and self._corpus is not None
        ):
            self.change_model_status(ModelType.TTS, ModelStatus.LOADED)
        else:
            self.change_model_status(ModelType.TTS, ModelStatus.FAILED)

    def unload(self):
        if self.model_status is ModelStatus.LOADING:
            return
        self.change_model_status(ModelType.TTS, ModelStatus.LOADING)
        self.logger.debug("Unloading")
        self._unload_model()
        self._unload_processor()
        self._unload_vocoder()
        self._unload_speaker_embeddings()
        self._unload_tokenizer()
        self.change_model_status(ModelType.TTS, ModelStatus.UNLOADED)

    def _load_model(self):
        model_class_ = self._model_class_
        if model_class_ is None:
            return
        self.logger.debug(f"Loading model {self.model_path}")
        try:
            self._model = model_class_.from_pretrained(
                self.model_path,
                local_files_only=True,
                torch_dtype=self.torch_dtype,
                device_map=self.device
            )
        except EnvironmentError as _e:
            self.logger.error(f"Failed to load model {_e}")

    def _load_tokenizer(self):
        self.logger.debug("Loading tokenizer")

        try:
            self._tokenizer = AutoTokenizer.from_pretrained(
                self.model_path,
                device_map=self.device,
                torch_dtype=self.torch_dtype,
                local_files_only=True,
                trust_remote_code=False
            )
        except Exception as e:
            self.logger.error("Failed to load tokenizer")
            self.logger.error(e)

    def _load_vocoder(self):
        self.logger.debug(f"Loading Vocoder {self.vocoder_path}")
        from transformers import SpeechT5HifiGan
        try:
            self._vocoder = self._vocoder = SpeechT5HifiGan.from_pretrained(
                self.vocoder_path,
                local_files_only=True,
                torch_dtype=self.torch_dtype,
                device_map=self.device
            )
        except Exception as e:
            self.logger.error("Failed to load vocoder")
            self.logger.error(e)

    def _load_processor(self):
        self.logger.debug("Loading Procesor")
        processor_class_ = self._processor_class_
        if processor_class_:
            try:
                processor = processor_class_.from_pretrained(
                    self.processor_path,
                    local_files_only=True,
                    torch_dtype=self.torch_dtype,
                    device_map=self.device
                )
                self._processor = processor
            except Exception as e:
                self.logger.error("Failed to load processor")
                self.logger.error(e)

    def _load_corpus(self):
        if self._input_text:
            self.logger.debug("Loading Corpus")
            corpus = open(self._input_text, "r").read()
            for key, value in self._character_replacement_map.items():
                corpus = corpus.replace(key, value)
            self._corpus = corpus.split(" ")

    def _load_speaker_embeddings(self):
        self.logger.debug("Loading speaker embeddings...")

        try:
            self._speaker_embeddings = torch.load(
                self.speaker_embeddings_path
            )
            if self.use_cuda and self._speaker_embeddings is not None:
                self._speaker_embeddings = self._speaker_embeddings.to(torch.bfloat16).cuda()  # Change to bfloat16

        except Exception as e:
            self.logger.error("Failed to load speaker embeddings")
            self.logger.error(e)

    def _unload_speaker_embeddings(self):
        self._speaker_embeddings = None
        clear_memory(self.memory_settings.default_gpu_tts)

    def _do_generate(self, message):
        self.logger.debug("Generating text-to-speech with T5")
        text = self._prepare_text(message)

        if text == "":
            return None

        self.logger.debug("Processing inputs...")

        inputs = self._processor(
            text=text,
            return_tensors="pt",
            torch_dtype=torch.float16  # Ensure inputs are in float16
        )
        inputs = self._move_inputs_to_device(inputs)

        self.logger.debug("Generating speech...")
        start = time.time()
        self._speaker_embeddings = self._speaker_embeddings.to(torch.float16).to(self.device)
        self._vocoder = self._vocoder.to(torch.float16).to(self.device)

        try:
            speech = self._model.generate(
                **inputs,
                speaker_embeddings=self._speaker_embeddings,
                vocoder=self._vocoder,
                max_length=100
            )
        except Exception as e:
            self.logger.error("Failed to generate speech")
            self.logger.error(e)
            self._cancel_generated_speech = False
            return None

        if not self._cancel_generated_speech:
            self.logger.debug("Generated speech in " + str(time.time() - start) + " seconds")
            response = speech.cpu().float().numpy()
            self.emit_signal(SignalCode.PROCESS_SPEECH_SIGNAL, {
                "message": text,
                "role": LLMChatRole.ASSISTANT
            })
            return response
        if not self._do_interrupt:
            self.logger.debug("Skipping generated speech: " + text)
            self._cancel_generated_speech = False
        return None

    def _move_inputs_to_device(self, inputs):
        use_cuda = self.tts_settings.use_cuda
        if use_cuda:
            self.logger.debug("Moving inputs to CUDA")
            try:
                for key in ("input_ids", "attention_mask"):
                    inputs[key] = inputs[key].to(self.device)

                if "history_prompt" in inputs:
                    for key in ("semantic_prompt", "coarse_prompt", "fine_prompt"):
                        inputs["history_prompt"][key] = inputs["history_prompt"][key].to(self.device)
            except AttributeError as e:
                self.logger.error("Failed to move inputs to CUDA")
                self.logger.error(e)
        return inputs

    def _unload_model(self):
        self._model = None
        self._current_model = None
        clear_memory(self.memory_settings.default_gpu_tts)

    def _unload_processor(self):
        self._processor = None
        clear_memory(self.memory_settings.default_gpu_tts)

    def _unload_vocoder(self):
        self._vocoder = None
        clear_memory(self.memory_settings.default_gpu_tts)

    def _unload_tokenizer(self):
        self.tokenizer = None
        clear_memory(self.memory_settings.default_gpu_tts)

    def unblock_tts_generator_signal(self):
        self.logger.debug("Unblocking text-to-speech generation...")
        self._do_interrupt = False
        self._paused = False

    def interrupt_process_signal(self):
        self._do_interrupt = True
        self._cancel_generated_speech = False
        self._paused = True
        self._text_queue = Queue()

    def _prepare_text(self, text) -> str:
        text = self._replace_unspeakable_characters(text)
        text = self._strip_emoji_characters(text)
        # the following function is currently disabled because we must first find a
        # reliable way to handle the word "I" and distinguish it from the Roman numeral "I"
        # text = self._roman_to_int(text)
        text = self._replace_numbers_with_words(text)
        return text

    @staticmethod
    def _replace_unspeakable_characters(text) -> str:
        # Replace ellipsis and other unspeakable characters
        text = text.replace("...", " ")
        text = text.replace("…", " ")
        text = text.replace("“", "")
        text = text.replace("”", "")
        text = text.replace("–", "")
        text = text.replace("—", "")
        text = text.replace('"', "")
        text = text.replace("-", "")
        text = text.replace("-", "")

        # Replace windows newlines
        text = text.replace("\r\n", " ")

        # Replace newlines
        text = text.replace("\n", " ")

        # Replace tabs
        text = text.replace("\t", " ")

        # Remove single quotes used as quotes but keep apostrophes
        text = re.sub(r"(?<=\W)'|'(?=\W)", "", text)
        text = re.sub(r"‘|’", "", text)

        return text

    @staticmethod
    def _strip_emoji_characters(text) -> str:
        # strip emojis
        emoji_pattern = re.compile(
            "["
            "\U0001F600-\U0001F64F"  # emoticons
            "\U0001F300-\U0001F5FF"  # symbols & pictographs
            "\U0001F680-\U0001F6FF"  # transport & map symbols
            "\U0001F700-\U0001F77F"  # alchemical symbols
            "\U0001F780-\U0001F7FF"  # Geometric Shapes Extended
            "\U0001F800-\U0001F8FF"  # Supplemental Arrows-C
            "\U0001F900-\U0001F9FF"  # Supplemental Symbols and Pictographs
            "\U0001FA00-\U0001FA6F"  # Chess Symbols
            "\U0001FA70-\U0001FAFF"  # Symbols and Pictographs Extended-A
            "\U00002702-\U000027B0"  # Dingbats
            "\U000024C2-\U0001F251"
            "]+", flags=re.UNICODE
        )
        text = emoji_pattern.sub(r'', text)
        return text

    @staticmethod
    def _roman_to_int(text) -> str:
        roman_numerals = {
            'I': 1, 'V': 5, 'X': 10, 'L': 50, 'C': 100, 'D': 500, 'M': 1000
        }

        def convert_roman_to_int(roman):
            total = 0
            prev_value = 0
            for char in reversed(roman):
                value = roman_numerals[char]
                if value < prev_value:
                    total -= value
                else:
                    total += value
                prev_value = value
            return str(total)

        # Replace Roman numerals with their integer values
        result = re.sub(r'\b[IVXLCDM]+\b', lambda match: convert_roman_to_int(match.group(0)), text)
        return result

    @staticmethod
    def _replace_numbers_with_words(text) -> str:
        p = inflect.engine()

        # Handle time formats separately
        text = re.sub(r'(\d+):(\d+)([APap][Mm])', lambda m: f"{p.number_to_words(m.group(1))} {p.number_to_words(m.group(2)).replace('zero', '').replace('-', ' ')} {m.group(3)[0].upper()} {m.group(3)[1].upper()}", text)
        text = re.sub(r'(\d+):(\d+)', lambda m: f"{p.number_to_words(m.group(1))} {p.number_to_words(m.group(2)).replace('-', ' ')}", text)

        # Split text into words and non-word characters
        words = re.findall(r'\d+|\D+', text)

        for i in range(len(words)):
            if words[i].isdigit():  # check if the word is a digit
                words[i] = p.number_to_words(words[i]).replace('-', ' ')

        # Join words with a space to ensure proper spacing
        result = ' '.join(words).replace('  ', ' ')

        # Ensure "PM" and "AM" are correctly spaced
        result = re.sub(r'\b([AP])M\b', r'\1 M', result)

        return result



from airunner.enums import SignalCode
from airunner.widgets.base_widget import BaseWidget
from airunner.widgets.facehugger.nullscream.templates.nullscream_ui import Ui_nullscream
from airunner.widgets.llm.templates.message_ui import Ui_message

from PySide6.QtCore import Signal, Slot


class NullscreamWidget(BaseWidget):
    widget_class_ = Ui_nullscream

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.register(
            SignalCode.APPLICATION_MAIN_WINDOW_LOADED_SIGNAL,
            self.on_application_main_window_loaded_signal
        )
        self.__data: dict = {}
        self.__filter_blocked = ""
        self.__filter_allowed = ""
        self.__formatted_blocked = ""
        self.__formatted_allowed = ""

    @Slot(str)
    def filter_blocked(self, val: str):
        self.ui.blocked.setPlainText(val)
        self.__filter_blocked = val
        self.display_lists()

    @Slot(str)
    def filter_allowed(self, val: str):
        self.ui.allowed.setPlainText(val)
        self.__filter_allowed = val
        self.display_lists()

    @property
    def data(self):
        data = self.__data.copy()
        if self.__filter_blocked != "":
            data["blocked"] = {k: v for k, v in data["blocked"].items() if self.__filter_blocked in k}
        if self.__filter_allowed != "":
            data["allowed"] = {k: v for k, v in data["allowed"].items() if self.__filter_allowed in k}
        return data

    def on_application_main_window_loaded_signal(self, data: dict):
        self.__data = data["main_window"].defendatron.nullscream_tracker.data
        self.display_lists()

    def display_lists(self):
        self.clear_lists()
        for root, data in self.data["allowed"].items():
            total = data["total"]
            self.__formatted_allowed += f"{root} total: {total}\n"
            for module in data["modules"]:
                if "airunner" not in module:
                    self.__formatted_allowed += f"    {module}\n"
        for root, data in self.data["blocked"].items():
            total = data["total"]
            self.__formatted_blocked += f"{root} total: {total}\n"
            for module in data["modules"]:
                if "airunner" not in module:
                    self.__formatted_blocked += f"    {module}\n"
        self.ui.allowed.setText(self.__formatted_allowed)
        self.ui.blocked.setText(self.__formatted_blocked)

    def clear_lists(self):
        self.__formatted_blocked = ""
        self.__formatted_allowed = ""
        self.ui.blocked.clear()
        self.ui.allowed.clear()
        self.ui.blocked.setPlainText("")
        self.ui.allowed.setPlainText("")



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'prompt_template_editor.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QGridLayout, QGroupBox,
    QHBoxLayout, QLabel, QPlainTextEdit, QPushButton,
    QSizePolicy, QWidget)

class Ui_Form(object):
    def setupUi(self, Form):
        if not Form.objectName():
            Form.setObjectName(u"Form")
        Form.resize(364, 447)
        self.gridLayout = QGridLayout(Form)
        self.gridLayout.setObjectName(u"gridLayout")
        self.groupBox_3 = QGroupBox(Form)
        self.groupBox_3.setObjectName(u"groupBox_3")
        self.gridLayout_4 = QGridLayout(self.groupBox_3)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.system_instructions = QPlainTextEdit(self.groupBox_3)
        self.system_instructions.setObjectName(u"system_instructions")

        self.gridLayout_4.addWidget(self.system_instructions, 1, 0, 1, 1)

        self.label = QLabel(self.groupBox_3)
        self.label.setObjectName(u"label")

        self.gridLayout_4.addWidget(self.label, 0, 0, 1, 1)


        self.gridLayout.addWidget(self.groupBox_3, 2, 0, 1, 1)

        self.groupBox = QGroupBox(Form)
        self.groupBox.setObjectName(u"groupBox")
        self.gridLayout_2 = QGridLayout(self.groupBox)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.llm_category = QComboBox(self.groupBox)
        self.llm_category.setObjectName(u"llm_category")

        self.gridLayout_2.addWidget(self.llm_category, 0, 0, 1, 1)


        self.gridLayout.addWidget(self.groupBox, 0, 0, 1, 1)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.template_name = QComboBox(Form)
        self.template_name.setObjectName(u"template_name")

        self.horizontalLayout.addWidget(self.template_name)

        self.pushButton_2 = QPushButton(Form)
        self.pushButton_2.setObjectName(u"pushButton_2")

        self.horizontalLayout.addWidget(self.pushButton_2)

        self.pushButton_3 = QPushButton(Form)
        self.pushButton_3.setObjectName(u"pushButton_3")

        self.horizontalLayout.addWidget(self.pushButton_3)

        self.pushButton = QPushButton(Form)
        self.pushButton.setObjectName(u"pushButton")

        self.horizontalLayout.addWidget(self.pushButton)


        self.gridLayout.addLayout(self.horizontalLayout, 4, 0, 1, 1)

        self.groupBox_2 = QGroupBox(Form)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.gridLayout_3 = QGridLayout(self.groupBox_2)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.template_format = QComboBox(self.groupBox_2)
        self.template_format.setObjectName(u"template_format")

        self.gridLayout_3.addWidget(self.template_format, 0, 0, 1, 1)


        self.gridLayout.addWidget(self.groupBox_2, 1, 0, 1, 1)


        self.retranslateUi(Form)
        self.llm_category.currentTextChanged.connect(Form.llm_category_changed)
        self.template_format.currentTextChanged.connect(Form.model_changed)
        self.system_instructions.textChanged.connect(Form.system_instructions_changed)
        self.template_name.currentTextChanged.connect(Form.template_name_changed)
        self.pushButton_2.clicked.connect(Form.save_clicked)
        self.pushButton_3.clicked.connect(Form.delete_clicked)
        self.pushButton.clicked.connect(Form.new_clicked)

        QMetaObject.connectSlotsByName(Form)
    # setupUi

    def retranslateUi(self, Form):
        Form.setWindowTitle(QCoreApplication.translate("Form", u"Form", None))
        self.groupBox_3.setTitle(QCoreApplication.translate("Form", u"System Instructions", None))
        self.system_instructions.setPlaceholderText(QCoreApplication.translate("Form", u"EXAMPLE: You are {{ botname }}. You are having a conversation with {{ username }}. {{ username }} is the user and you are the assistant. You should stay in character and respond as {{ botname }}.", None))
        self.label.setText(QCoreApplication.translate("Form", u"Keep the instructions brief, but be descriptive.", None))
        self.groupBox.setTitle(QCoreApplication.translate("Form", u"LLM Category", None))
        self.pushButton_2.setText(QCoreApplication.translate("Form", u"Save", None))
        self.pushButton_3.setText(QCoreApplication.translate("Form", u"Delete", None))
        self.pushButton.setText(QCoreApplication.translate("Form", u"New", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("Form", u"Model", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'main_window.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QApplication, QGridLayout, QMainWindow, QMenu,
    QMenuBar, QSizePolicy, QSplitter, QStatusBar,
    QToolBar, QVBoxLayout, QWidget)

from airunner.widgets.canvas.canvas_widget import CanvasWidget
from airunner.widgets.generator_form.generator_form_widget import GeneratorForm
from airunner.widgets.tool_tab.tool_tab_widget import ToolTabWidget
import airunner.resources_light_rc
import airunner.resources_dark_rc

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(829, 736)
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        MainWindow.setMinimumSize(QSize(0, 0))
        font = QFont()
        font.setPointSize(8)
        MainWindow.setFont(font)
        icon = QIcon()
        icon.addFile(u":/icons/icon_256.png", QSize(), QIcon.Normal, QIcon.Off)
        MainWindow.setWindowIcon(icon)
        MainWindow.setStyleSheet(u"")
        self.actionNew = QAction(MainWindow)
        self.actionNew.setObjectName(u"actionNew")
        icon1 = QIcon(QIcon.fromTheme(u"document-new"))
        self.actionNew.setIcon(icon1)
        self.actionImport = QAction(MainWindow)
        self.actionImport.setObjectName(u"actionImport")
        icon2 = QIcon(QIcon.fromTheme(u"document-open"))
        self.actionImport.setIcon(icon2)
        self.actionExport = QAction(MainWindow)
        self.actionExport.setObjectName(u"actionExport")
        self.actionCopy = QAction(MainWindow)
        self.actionCopy.setObjectName(u"actionCopy")
        self.actionPaste = QAction(MainWindow)
        self.actionPaste.setObjectName(u"actionPaste")
        self.actionUndo = QAction(MainWindow)
        self.actionUndo.setObjectName(u"actionUndo")
        icon3 = QIcon(QIcon.fromTheme(u"edit-undo"))
        self.actionUndo.setIcon(icon3)
        self.actionRedo = QAction(MainWindow)
        self.actionRedo.setObjectName(u"actionRedo")
        icon4 = QIcon(QIcon.fromTheme(u"edit-redo"))
        self.actionRedo.setIcon(icon4)
        self.actionGrid = QAction(MainWindow)
        self.actionGrid.setObjectName(u"actionGrid")
        self.actionPreferences = QAction(MainWindow)
        self.actionPreferences.setObjectName(u"actionPreferences")
        self.actionCanvas_color = QAction(MainWindow)
        self.actionCanvas_color.setObjectName(u"actionCanvas_color")
        self.actionReset_Settings = QAction(MainWindow)
        self.actionReset_Settings.setObjectName(u"actionReset_Settings")
        self.actionResize_on_Paste = QAction(MainWindow)
        self.actionResize_on_Paste.setObjectName(u"actionResize_on_Paste")
        self.actionResize_on_Paste.setCheckable(True)
        self.actionMemory = QAction(MainWindow)
        self.actionMemory.setObjectName(u"actionMemory")
        self.actionAbout = QAction(MainWindow)
        self.actionAbout.setObjectName(u"actionAbout")
        self.actionCheck_for_updates = QAction(MainWindow)
        self.actionCheck_for_updates.setObjectName(u"actionCheck_for_updates")
        self.actionOpen = QAction(MainWindow)
        self.actionOpen.setObjectName(u"actionOpen")
        self.actionView = QAction(MainWindow)
        self.actionView.setObjectName(u"actionView")
        self.actionAdvanced = QAction(MainWindow)
        self.actionAdvanced.setObjectName(u"actionAdvanced")
        self.actionGaussian_Blur = QAction(MainWindow)
        self.actionGaussian_Blur.setObjectName(u"actionGaussian_Blur")
        self.actionGaussian_Blur.setEnabled(True)
        self.actionBox_Blur = QAction(MainWindow)
        self.actionBox_Blur.setObjectName(u"actionBox_Blur")
        self.actionUnsharp_Mask = QAction(MainWindow)
        self.actionUnsharp_Mask.setObjectName(u"actionUnsharp_Mask")
        self.actionPixel_Art = QAction(MainWindow)
        self.actionPixel_Art.setObjectName(u"actionPixel_Art")
        self.actionPixel_Art.setVisible(True)
        self.actionInvert = QAction(MainWindow)
        self.actionInvert.setObjectName(u"actionInvert")
        self.actionSaturation = QAction(MainWindow)
        self.actionSaturation.setObjectName(u"actionSaturation")
        self.actionSaturation.setVisible(True)
        self.actionColor_Balance = QAction(MainWindow)
        self.actionColor_Balance.setObjectName(u"actionColor_Balance")
        self.actionColor_Balance.setCheckable(False)
        self.actionColor_Balance.setEnabled(True)
        self.actionColor_Balance.setVisible(True)
        self.actionSave = QAction(MainWindow)
        self.actionSave.setObjectName(u"actionSave")
        self.actionLoad = QAction(MainWindow)
        self.actionLoad.setObjectName(u"actionLoad")
        self.actionQuit = QAction(MainWindow)
        self.actionQuit.setObjectName(u"actionQuit")
        self.actionPixel_Art_2 = QAction(MainWindow)
        self.actionPixel_Art_2.setObjectName(u"actionPixel_Art_2")
        self.actionBug_report = QAction(MainWindow)
        self.actionBug_report.setObjectName(u"actionBug_report")
        self.actionReport_vulnerability = QAction(MainWindow)
        self.actionReport_vulnerability.setObjectName(u"actionReport_vulnerability")
        self.actionDiscord = QAction(MainWindow)
        self.actionDiscord.setObjectName(u"actionDiscord")
        self.actionExtensions = QAction(MainWindow)
        self.actionExtensions.setObjectName(u"actionExtensions")
        self.actionShow_image_preview = QAction(MainWindow)
        self.actionShow_image_preview.setObjectName(u"actionShow_image_preview")
        self.actionShow_image_preview.setCheckable(True)
        self.actionImage_to_new_layer = QAction(MainWindow)
        self.actionImage_to_new_layer.setObjectName(u"actionImage_to_new_layer")
        self.actionImage_to_new_layer.setCheckable(True)
        self.actionAuto_export_images = QAction(MainWindow)
        self.actionAuto_export_images.setObjectName(u"actionAuto_export_images")
        self.actionAuto_export_images.setCheckable(True)
        self.actionImage_export_settings = QAction(MainWindow)
        self.actionImage_export_settings.setObjectName(u"actionImage_export_settings")
        self.actionCheck_for_latest_version_on_startup = QAction(MainWindow)
        self.actionCheck_for_latest_version_on_startup.setObjectName(u"actionCheck_for_latest_version_on_startup")
        self.actionCheck_for_latest_version_on_startup.setCheckable(True)
        self.actionModel_Merger = QAction(MainWindow)
        self.actionModel_Merger.setObjectName(u"actionModel_Merger")
        self.actionSaturation_Filter = QAction(MainWindow)
        self.actionSaturation_Filter.setObjectName(u"actionSaturation_Filter")
        self.actionColor_Balance_2 = QAction(MainWindow)
        self.actionColor_Balance_2.setObjectName(u"actionColor_Balance_2")
        self.actionRGB_Noise = QAction(MainWindow)
        self.actionRGB_Noise.setObjectName(u"actionRGB_Noise")
        self.actionGaussian_Blur_2 = QAction(MainWindow)
        self.actionGaussian_Blur_2.setObjectName(u"actionGaussian_Blur_2")
        self.actionBox_Blur_2 = QAction(MainWindow)
        self.actionBox_Blur_2.setObjectName(u"actionBox_Blur_2")
        self.actionRotate_90_clockwise = QAction(MainWindow)
        self.actionRotate_90_clockwise.setObjectName(u"actionRotate_90_clockwise")
        self.actionRotate_90_counter_clockwise = QAction(MainWindow)
        self.actionRotate_90_counter_clockwise.setObjectName(u"actionRotate_90_counter_clockwise")
        self.actionSave_prompt = QAction(MainWindow)
        self.actionSave_prompt.setObjectName(u"actionSave_prompt")
        self.actionShow_Active_Image_Area = QAction(MainWindow)
        self.actionShow_Active_Image_Area.setObjectName(u"actionShow_Active_Image_Area")
        self.actionShow_Active_Image_Area.setCheckable(True)
        self.actionImage_Interpolation = QAction(MainWindow)
        self.actionImage_Interpolation.setObjectName(u"actionImage_Interpolation")
        self.actionImage_interpolation = QAction(MainWindow)
        self.actionImage_interpolation.setObjectName(u"actionImage_interpolation")
        self.actionFilm = QAction(MainWindow)
        self.actionFilm.setObjectName(u"actionFilm")
        self.actionDeterministic_generation = QAction(MainWindow)
        self.actionDeterministic_generation.setObjectName(u"actionDeterministic_generation")
        self.actionDark_mode = QAction(MainWindow)
        self.actionDark_mode.setObjectName(u"actionDark_mode")
        self.actionDark_mode.setCheckable(True)
        self.actionConsole_window = QAction(MainWindow)
        self.actionConsole_window.setObjectName(u"actionConsole_window")
        self.actionPrompt_Browser = QAction(MainWindow)
        self.actionPrompt_Browser.setObjectName(u"actionPrompt_Browser")
        self.actionClear_all_prompts = QAction(MainWindow)
        self.actionClear_all_prompts.setObjectName(u"actionClear_all_prompts")
        self.actionUndo_2 = QAction(MainWindow)
        self.actionUndo_2.setObjectName(u"actionUndo_2")
        icon5 = QIcon()
        icon5.addFile(u"../../../../../../../../../../../../../../../../../../../.designer/src/icons/007-undo.png", QSize(), QIcon.Normal, QIcon.Off)
        self.actionUndo_2.setIcon(icon5)
        self.actionRedo_2 = QAction(MainWindow)
        self.actionRedo_2.setObjectName(u"actionRedo_2")
        icon6 = QIcon()
        icon6.addFile(u"../../../../../../../../../../../../../../../../../../../.designer/src/icons/008-redo.png", QSize(), QIcon.Normal, QIcon.Off)
        self.actionRedo_2.setIcon(icon6)
        self.actionCut = QAction(MainWindow)
        self.actionCut.setObjectName(u"actionCut")
        self.actionHalftone = QAction(MainWindow)
        self.actionHalftone.setObjectName(u"actionHalftone")
        self.actionQuick_Export = QAction(MainWindow)
        self.actionQuick_Export.setObjectName(u"actionQuick_Export")
        self.actionRegistration_error = QAction(MainWindow)
        self.actionRegistration_error.setObjectName(u"actionRegistration_error")
        self.actionHuggingface_Cache_manager = QAction(MainWindow)
        self.actionHuggingface_Cache_manager.setObjectName(u"actionHuggingface_Cache_manager")
        self.actionModel_Manager = QAction(MainWindow)
        self.actionModel_Manager.setObjectName(u"actionModel_Manager")
        self.actionModel_Manager_2 = QAction(MainWindow)
        self.actionModel_Manager_2.setObjectName(u"actionModel_Manager_2")
        self.actionControlNet = QAction(MainWindow)
        self.actionControlNet.setObjectName(u"actionControlNet")
        self.actionPrompt_Builder = QAction(MainWindow)
        self.actionPrompt_Builder.setObjectName(u"actionPrompt_Builder")
        self.actionEmbeddings = QAction(MainWindow)
        self.actionEmbeddings.setObjectName(u"actionEmbeddings")
        self.actionLoRA = QAction(MainWindow)
        self.actionLoRA.setObjectName(u"actionLoRA")
        self.actionPen = QAction(MainWindow)
        self.actionPen.setObjectName(u"actionPen")
        self.actionStableDiffusion = QAction(MainWindow)
        self.actionStableDiffusion.setObjectName(u"actionStableDiffusion")
        self.actionKandinsky = QAction(MainWindow)
        self.actionKandinsky.setObjectName(u"actionKandinsky")
        self.actionShap_E = QAction(MainWindow)
        self.actionShap_E.setObjectName(u"actionShap_E")
        self.actionRun_setup_wizard = QAction(MainWindow)
        self.actionRun_setup_wizard.setObjectName(u"actionRun_setup_wizard")
        self.actionBrowse_AI_Runner_Path = QAction(MainWindow)
        self.actionBrowse_AI_Runner_Path.setObjectName(u"actionBrowse_AI_Runner_Path")
        self.actionBrowse_Images_Path = QAction(MainWindow)
        self.actionBrowse_Images_Path.setObjectName(u"actionBrowse_Images_Path")
        self.actionBrowse_Images_Path_2 = QAction(MainWindow)
        self.actionBrowse_Images_Path_2.setObjectName(u"actionBrowse_Images_Path_2")
        self.actionBrowse_Gifs_Path = QAction(MainWindow)
        self.actionBrowse_Gifs_Path.setObjectName(u"actionBrowse_Gifs_Path")
        self.actionTxt2img_Img2img_Models = QAction(MainWindow)
        self.actionTxt2img_Img2img_Models.setObjectName(u"actionTxt2img_Img2img_Models")
        self.actionInpaint_Outpaint_Models = QAction(MainWindow)
        self.actionInpaint_Outpaint_Models.setObjectName(u"actionInpaint_Outpaint_Models")
        self.actionActive_Grid = QAction(MainWindow)
        self.actionActive_Grid.setObjectName(u"actionActive_Grid")
        self.actionDeterministic_Batches = QAction(MainWindow)
        self.actionDeterministic_Batches.setObjectName(u"actionDeterministic_Batches")
        self.actionStandard_Batches = QAction(MainWindow)
        self.actionStandard_Batches.setObjectName(u"actionStandard_Batches")
        self.actionLLM_beta = QAction(MainWindow)
        self.actionLLM_beta.setObjectName(u"actionLLM_beta")
        self.actionReset_Settings_2 = QAction(MainWindow)
        self.actionReset_Settings_2.setObjectName(u"actionReset_Settings_2")
        self.actionImage_Browser = QAction(MainWindow)
        self.actionImage_Browser.setObjectName(u"actionImage_Browser")
        self.actionModel_Manager_3 = QAction(MainWindow)
        self.actionModel_Manager_3.setObjectName(u"actionModel_Manager_3")
        self.actionImport_input_image = QAction(MainWindow)
        self.actionImport_input_image.setObjectName(u"actionImport_input_image")
        self.actionExport_input_image = QAction(MainWindow)
        self.actionExport_input_image.setObjectName(u"actionExport_input_image")
        self.actionImport_2 = QAction(MainWindow)
        self.actionImport_2.setObjectName(u"actionImport_2")
        self.actionExport_image = QAction(MainWindow)
        self.actionExport_image.setObjectName(u"actionExport_image")
        self.actionImport_Image = QAction(MainWindow)
        self.actionImport_Image.setObjectName(u"actionImport_Image")
        self.actionExport_Image = QAction(MainWindow)
        self.actionExport_Image.setObjectName(u"actionExport_Image")
        self.actionImport_3 = QAction(MainWindow)
        self.actionImport_3.setObjectName(u"actionImport_3")
        self.actionExport_2 = QAction(MainWindow)
        self.actionExport_2.setObjectName(u"actionExport_2")
        self.controlnet_import = QAction(MainWindow)
        self.controlnet_import.setObjectName(u"controlnet_import")
        self.controlnet_export = QAction(MainWindow)
        self.controlnet_export.setObjectName(u"controlnet_export")
        self.drawingpad_import = QAction(MainWindow)
        self.drawingpad_import.setObjectName(u"drawingpad_import")
        self.drawingpad_export = QAction(MainWindow)
        self.drawingpad_export.setObjectName(u"drawingpad_export")
        self.actionGenerate_while_drawing = QAction(MainWindow)
        self.actionGenerate_while_drawing.setObjectName(u"actionGenerate_while_drawing")
        self.actionGenerate_while_drawing.setCheckable(True)
        self.actionOverlay_input_image = QAction(MainWindow)
        self.actionOverlay_input_image.setObjectName(u"actionOverlay_input_image")
        self.actionOverlay_input_image.setCheckable(True)
        self.enable_controlnet = QAction(MainWindow)
        self.enable_controlnet.setObjectName(u"enable_controlnet")
        self.enable_controlnet.setCheckable(True)
        self.actionUnload_LLm = QAction(MainWindow)
        self.actionUnload_LLm.setObjectName(u"actionUnload_LLm")
        self.actionUnload_Image_model = QAction(MainWindow)
        self.actionUnload_Image_model.setObjectName(u"actionUnload_Image_model")
        self.actionClear_Memory = QAction(MainWindow)
        self.actionClear_Memory.setObjectName(u"actionClear_Memory")
        self.actionSafety_Checker = QAction(MainWindow)
        self.actionSafety_Checker.setObjectName(u"actionSafety_Checker")
        self.actionSafety_Checker.setCheckable(True)
        self.actionSafety_Checker.setChecked(False)
        self.outpaint_import = QAction(MainWindow)
        self.outpaint_import.setObjectName(u"outpaint_import")
        self.outpaint_export = QAction(MainWindow)
        self.outpaint_export.setObjectName(u"outpaint_export")
        self.outpaint_enabled = QAction(MainWindow)
        self.outpaint_enabled.setObjectName(u"outpaint_enabled")
        self.outpaint_enabled.setCheckable(True)
        self.actionModel_Manager_4 = QAction(MainWindow)
        self.actionModel_Manager_4.setObjectName(u"actionModel_Manager_4")
        self.actionRun_setup_wizard_2 = QAction(MainWindow)
        self.actionRun_setup_wizard_2.setObjectName(u"actionRun_setup_wizard_2")
        self.actionStats = QAction(MainWindow)
        self.actionStats.setObjectName(u"actionStats")
        self.actionSettings = QAction(MainWindow)
        self.actionSettings.setObjectName(u"actionSettings")
        icon7 = QIcon(QIcon.fromTheme(u"preferences-system"))
        self.actionSettings.setIcon(icon7)
        self.actionToggle_LLM = QAction(MainWindow)
        self.actionToggle_LLM.setObjectName(u"actionToggle_LLM")
        self.actionToggle_LLM.setCheckable(True)
        self.actionToggle_LLM.setChecked(False)
        icon8 = QIcon()
        iconThemeName = u"applications-internet"
        if QIcon.hasThemeIcon(iconThemeName):
            icon8 = QIcon.fromTheme(iconThemeName)
        else:
            icon8.addFile(u"../../../../../../../.designer/backup/applications-internet", QSize(), QIcon.Normal, QIcon.On)

        self.actionToggle_LLM.setIcon(icon8)
        self.actionToggle_Text_to_Speech = QAction(MainWindow)
        self.actionToggle_Text_to_Speech.setObjectName(u"actionToggle_Text_to_Speech")
        self.actionToggle_Text_to_Speech.setCheckable(True)
        icon9 = QIcon(QIcon.fromTheme(u"audio-volume-high"))
        self.actionToggle_Text_to_Speech.setIcon(icon9)
        self.actionToggle_Speech_to_Text = QAction(MainWindow)
        self.actionToggle_Speech_to_Text.setObjectName(u"actionToggle_Speech_to_Text")
        self.actionToggle_Speech_to_Text.setCheckable(True)
        icon10 = QIcon()
        iconThemeName = u"audio-input-microphone"
        if QIcon.hasThemeIcon(iconThemeName):
            icon10 = QIcon.fromTheme(iconThemeName)
        else:
            icon10.addFile(u"../../../../../../../.designer/backup/audio-input-microphone", QSize(), QIcon.Normal, QIcon.On)

        self.actionToggle_Speech_to_Text.setIcon(icon10)
        self.actionToggle_Stable_Diffusion = QAction(MainWindow)
        self.actionToggle_Stable_Diffusion.setObjectName(u"actionToggle_Stable_Diffusion")
        self.actionToggle_Stable_Diffusion.setCheckable(True)
        icon11 = QIcon(QIcon.fromTheme(u"applications-graphics"))
        self.actionToggle_Stable_Diffusion.setIcon(icon11)
        self.actionToggle_Controlnet = QAction(MainWindow)
        self.actionToggle_Controlnet.setObjectName(u"actionToggle_Controlnet")
        self.actionToggle_Controlnet.setCheckable(True)
        self.actionToggle_Controlnet.setChecked(False)
        icon12 = QIcon(QIcon.fromTheme(u"insert-image"))
        self.actionToggle_Controlnet.setIcon(icon12)
        self.actionExport_3 = QAction(MainWindow)
        self.actionExport_3.setObjectName(u"actionExport_3")
        icon13 = QIcon(QIcon.fromTheme(u"document-save"))
        self.actionExport_3.setIcon(icon13)
        self.actionToggle_Active_Grid_Area = QAction(MainWindow)
        self.actionToggle_Active_Grid_Area.setObjectName(u"actionToggle_Active_Grid_Area")
        self.actionToggle_Active_Grid_Area.setCheckable(True)
        icon14 = QIcon()
        icon14.addFile(u":/icons/light/object-selected-icon.svg", QSize(), QIcon.Normal, QIcon.Off)
        self.actionToggle_Active_Grid_Area.setIcon(icon14)
        self.actionToggle_Selection = QAction(MainWindow)
        self.actionToggle_Selection.setObjectName(u"actionToggle_Selection")
        self.actionToggle_Selection.setCheckable(True)
        icon15 = QIcon()
        icon15.addFile(u":/icons/light/select-svgrepo-com.svg", QSize(), QIcon.Normal, QIcon.Off)
        self.actionToggle_Selection.setIcon(icon15)
        self.actionToggle_Brush = QAction(MainWindow)
        self.actionToggle_Brush.setObjectName(u"actionToggle_Brush")
        self.actionToggle_Brush.setCheckable(True)
        icon16 = QIcon()
        icon16.addFile(u":/icons/light/pencil-icon.svg", QSize(), QIcon.Normal, QIcon.Off)
        self.actionToggle_Brush.setIcon(icon16)
        self.actionToggle_Grid = QAction(MainWindow)
        self.actionToggle_Grid.setObjectName(u"actionToggle_Grid")
        self.actionToggle_Grid.setCheckable(True)
        icon17 = QIcon()
        icon17.addFile(u":/icons/light/frame-grid-icon.svg", QSize(), QIcon.Normal, QIcon.Off)
        self.actionToggle_Grid.setIcon(icon17)
        self.actionRecenter = QAction(MainWindow)
        self.actionRecenter.setObjectName(u"actionRecenter")
        icon18 = QIcon()
        icon18.addFile(u":/icons/light/circle-center-icon.svg", QSize(), QIcon.Normal, QIcon.Off)
        self.actionRecenter.setIcon(icon18)
        self.actionToggle_Eraser = QAction(MainWindow)
        self.actionToggle_Eraser.setObjectName(u"actionToggle_Eraser")
        self.actionToggle_Eraser.setCheckable(True)
        icon19 = QIcon()
        icon19.addFile(u":/icons/light/eraser-icon.svg", QSize(), QIcon.Normal, QIcon.Off)
        self.actionToggle_Eraser.setIcon(icon19)
        self.actionMask_toggle = QAction(MainWindow)
        self.actionMask_toggle.setObjectName(u"actionMask_toggle")
        self.actionMask_toggle.setCheckable(True)
        icon20 = QIcon()
        icon20.addFile(u":/icons/light/layer-icon.svg", QSize(), QIcon.Normal, QIcon.Off)
        self.actionMask_toggle.setIcon(icon20)
        self.actionMask_toggle.setMenuRole(QAction.MenuRole.NoRole)
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.centralwidget.setEnabled(True)
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.centralwidget.setMinimumSize(QSize(0, 0))
        self.centralwidget.setLayoutDirection(Qt.LayoutDirection.LeftToRight)
        self.gridLayout_3 = QGridLayout(self.centralwidget)
        self.gridLayout_3.setSpacing(0)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.mode_tab_widget = QWidget(self.centralwidget)
        self.mode_tab_widget.setObjectName(u"mode_tab_widget")
        self.mode_tab_widget.setAutoFillBackground(False)
        self.mode_tab_widget.setStyleSheet(u"QTabWidget#mode_tab_widget::pane { border: 0; background: transparent; }")
        self.gridLayout = QGridLayout(self.mode_tab_widget)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(0, 1, 0, 0)
        self.content_splitter = QSplitter(self.mode_tab_widget)
        self.content_splitter.setObjectName(u"content_splitter")
        self.content_splitter.setOrientation(Qt.Orientation.Horizontal)
        self.generator_widget = GeneratorForm(self.content_splitter)
        self.generator_widget.setObjectName(u"generator_widget")
        self.content_splitter.addWidget(self.generator_widget)
        self.canvas_widget = QWidget(self.content_splitter)
        self.canvas_widget.setObjectName(u"canvas_widget")
        self.canvas_widget.setMinimumSize(QSize(0, 0))
        self.gridLayout_2 = QGridLayout(self.canvas_widget)
        self.gridLayout_2.setSpacing(0)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.canvas_widget_2 = CanvasWidget(self.canvas_widget)
        self.canvas_widget_2.setObjectName(u"canvas_widget_2")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Ignored, QSizePolicy.Policy.MinimumExpanding)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.canvas_widget_2.sizePolicy().hasHeightForWidth())
        self.canvas_widget_2.setSizePolicy(sizePolicy1)
        self.canvas_widget_2.setMinimumSize(QSize(0, 0))
        self.canvas_widget_2.setMaximumSize(QSize(16777215, 16777215))
        self.canvas_widget_2.setCursor(QCursor(Qt.ArrowCursor))
        self.canvas_widget_2.setAcceptDrops(True)

        self.gridLayout_2.addWidget(self.canvas_widget_2, 0, 0, 1, 1)

        self.content_splitter.addWidget(self.canvas_widget)
        self.panel_container = QWidget(self.content_splitter)
        self.panel_container.setObjectName(u"panel_container")
        self.verticalLayout_2 = QVBoxLayout(self.panel_container)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.tool_tab_widget = ToolTabWidget(self.panel_container)
        self.tool_tab_widget.setObjectName(u"tool_tab_widget")

        self.verticalLayout_2.addWidget(self.tool_tab_widget)

        self.content_splitter.addWidget(self.panel_container)

        self.gridLayout.addWidget(self.content_splitter, 0, 0, 1, 1)


        self.gridLayout_3.addWidget(self.mode_tab_widget, 0, 0, 1, 1)

        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 829, 22))
        font1 = QFont()
        font1.setPointSize(11)
        self.menubar.setFont(font1)
        self.menuFile = QMenu(self.menubar)
        self.menuFile.setObjectName(u"menuFile")
        self.menuFile.setTearOffEnabled(False)
        self.menuFile.setSeparatorsCollapsible(False)
        self.menuEdit = QMenu(self.menubar)
        self.menuEdit.setObjectName(u"menuEdit")
        self.menuFilters = QMenu(self.menubar)
        self.menuFilters.setObjectName(u"menuFilters")
        self.menuAbout = QMenu(self.menubar)
        self.menuAbout.setObjectName(u"menuAbout")
        self.menuAbout.setTearOffEnabled(False)
        self.menuImage = QMenu(self.menubar)
        self.menuImage.setObjectName(u"menuImage")
        self.menuView = QMenu(self.menubar)
        self.menuView.setObjectName(u"menuView")
        self.menuTools = QMenu(self.menubar)
        self.menuTools.setObjectName(u"menuTools")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.toolBar = QToolBar(MainWindow)
        self.toolBar.setObjectName(u"toolBar")
        self.toolBar.setMovable(True)
        self.toolBar.setIconSize(QSize(18, 18))
        self.toolBar.setFloatable(False)
        MainWindow.addToolBar(Qt.ToolBarArea.TopToolBarArea, self.toolBar)
        self.model_toolbar = QToolBar(MainWindow)
        self.model_toolbar.setObjectName(u"model_toolbar")
        self.model_toolbar.setLayoutDirection(Qt.LayoutDirection.RightToLeft)
        self.model_toolbar.setMovable(True)
        self.model_toolbar.setOrientation(Qt.Orientation.Horizontal)
        self.model_toolbar.setIconSize(QSize(18, 18))
        self.model_toolbar.setFloatable(False)
        MainWindow.addToolBar(Qt.ToolBarArea.BottomToolBarArea, self.model_toolbar)

        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuEdit.menuAction())
        self.menubar.addAction(self.menuView.menuAction())
        self.menubar.addAction(self.menuImage.menuAction())
        self.menubar.addAction(self.menuFilters.menuAction())
        self.menubar.addAction(self.menuTools.menuAction())
        self.menubar.addAction(self.menuAbout.menuAction())
        self.menuFile.addAction(self.actionNew)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionImport)
        self.menuFile.addAction(self.actionExport_3)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionReset_Settings_2)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionQuit)
        self.menuEdit.addSeparator()
        self.menuEdit.addAction(self.actionCut)
        self.menuEdit.addAction(self.actionCopy)
        self.menuEdit.addAction(self.actionPaste)
        self.menuEdit.addSeparator()
        self.menuEdit.addAction(self.actionRotate_90_clockwise)
        self.menuEdit.addAction(self.actionRotate_90_counter_clockwise)
        self.menuEdit.addSeparator()
        self.menuEdit.addAction(self.actionClear_all_prompts)
        self.menuEdit.addSeparator()
        self.menuEdit.addAction(self.actionSettings)
        self.menuFilters.addSeparator()
        self.menuAbout.addAction(self.actionAbout)
        self.menuAbout.addAction(self.actionBug_report)
        self.menuAbout.addAction(self.actionReport_vulnerability)
        self.menuImage.addSeparator()
        self.menuImage.addSeparator()
        self.menuImage.addAction(self.actionToggle_Active_Grid_Area)
        self.menuImage.addAction(self.actionToggle_Selection)
        self.menuImage.addAction(self.actionToggle_Brush)
        self.menuImage.addAction(self.actionToggle_Eraser)
        self.menuImage.addAction(self.actionToggle_Grid)
        self.menuView.addAction(self.actionPrompt_Browser)
        self.menuView.addSeparator()
        self.menuView.addAction(self.actionBrowse_AI_Runner_Path)
        self.menuView.addSeparator()
        self.menuView.addAction(self.actionBrowse_Images_Path_2)
        self.menuView.addSeparator()
        self.menuTools.addAction(self.actionSafety_Checker)
        self.menuTools.addAction(self.actionToggle_Stable_Diffusion)
        self.menuTools.addAction(self.actionToggle_Controlnet)
        self.menuTools.addSeparator()
        self.menuTools.addAction(self.actionToggle_LLM)
        self.menuTools.addAction(self.actionToggle_Text_to_Speech)
        self.menuTools.addAction(self.actionToggle_Speech_to_Text)
        self.menuTools.addSeparator()
        self.menuTools.addAction(self.actionStats)
        self.menuTools.addSeparator()
        self.menuTools.addAction(self.actionRun_setup_wizard_2)
        self.toolBar.addAction(self.actionNew)
        self.toolBar.addAction(self.actionImport)
        self.toolBar.addAction(self.actionExport_3)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionToggle_Active_Grid_Area)
        self.toolBar.addAction(self.actionToggle_Brush)
        self.toolBar.addAction(self.actionToggle_Eraser)
        self.toolBar.addAction(self.actionToggle_Grid)
        self.toolBar.addAction(self.actionMask_toggle)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionUndo)
        self.toolBar.addAction(self.actionRedo)
        self.model_toolbar.addAction(self.actionSettings)
        self.model_toolbar.addSeparator()
        self.model_toolbar.addAction(self.actionToggle_Speech_to_Text)
        self.model_toolbar.addAction(self.actionToggle_Text_to_Speech)
        self.model_toolbar.addAction(self.actionToggle_LLM)
        self.model_toolbar.addSeparator()
        self.model_toolbar.addAction(self.actionToggle_Controlnet)
        self.model_toolbar.addAction(self.actionToggle_Stable_Diffusion)

        self.retranslateUi(MainWindow)
        self.actionAbout.triggered.connect(MainWindow.action_show_about_window)
        self.actionExport.triggered.connect(MainWindow.action_export_image_triggered)
        self.actionNew.triggered.connect(MainWindow.action_new_document_triggered)
        self.actionImport.triggered.connect(MainWindow.action_import_image_triggered)
        self.actionQuit.triggered.connect(MainWindow.action_quit_triggered)
        self.actionPaste.triggered.connect(MainWindow.action_paste_image_triggered)
        self.actionCopy.triggered.connect(MainWindow.action_copy_image_triggered)
        self.actionCut.triggered.connect(MainWindow.action_cut_image_triggered)
        self.actionRotate_90_clockwise.triggered.connect(MainWindow.action_rotate_90_clockwise_triggered)
        self.actionRotate_90_counter_clockwise.triggered.connect(MainWindow.action_rotate_90_counterclockwise_triggered)
        self.actionPrompt_Browser.triggered.connect(MainWindow.action_show_prompt_browser_triggered)
        self.actionClear_all_prompts.triggered.connect(MainWindow.action_clear_all_prompts_triggered)
        self.actionBrowse_AI_Runner_Path.triggered.connect(MainWindow.action_triggered_browse_ai_runner_path)
        self.actionBrowse_Images_Path_2.triggered.connect(MainWindow.action_show_images_path)
        self.actionTxt2img_Img2img_Models.triggered.connect(MainWindow.action_show_model_path_txt2img)
        self.actionInpaint_Outpaint_Models.triggered.connect(MainWindow.action_show_model_path_inpaint)
        self.actionModel_Manager_2.triggered.connect(MainWindow.action_show_model_manager)
        self.actionBug_report.triggered.connect(MainWindow.action_open_bug_report)
        self.actionDiscord.triggered.connect(MainWindow.action_open_discord)
        self.actionReport_vulnerability.triggered.connect(MainWindow.action_open_vulnerability_report)
        self.actionReset_Settings_2.triggered.connect(MainWindow.action_reset_settings)
        self.actionModel_Manager_3.triggered.connect(MainWindow.action_show_model_manager)
        self.drawingpad_import.triggered.connect(MainWindow.import_drawingpad_image)
        self.drawingpad_export.triggered.connect(MainWindow.export_drawingpad_image)
        self.controlnet_import.triggered.connect(MainWindow.import_controlnet_image)
        self.controlnet_export.triggered.connect(MainWindow.export_controlnet_image)
        self.enable_controlnet.toggled.connect(MainWindow.action_controlnet_toggled)
        self.actionSafety_Checker.toggled.connect(MainWindow.action_toggle_nsfw_filter_triggered)
        self.outpaint_export.triggered.connect(MainWindow.action_outpaint_export)
        self.outpaint_enabled.toggled.connect(MainWindow.action_outpaint_toggled)
        self.outpaint_import.triggered.connect(MainWindow.action_outpaint_import)
        self.actionRun_setup_wizard_2.triggered.connect(MainWindow.action_run_setup_wizard_clicked)
        self.actionStats.triggered.connect(MainWindow.action_stats_triggered)
        self.actionSettings.triggered.connect(MainWindow.action_show_settings)
        self.actionToggle_Controlnet.toggled.connect(MainWindow.action_controlnet_toggled)
        self.actionToggle_Stable_Diffusion.toggled.connect(MainWindow.action_image_generator_toggled)
        self.actionToggle_Text_to_Speech.toggled.connect(MainWindow.tts_button_toggled)
        self.actionToggle_Speech_to_Text.toggled.connect(MainWindow.v2t_button_toggled)
        self.actionToggle_Grid.toggled.connect(MainWindow.action_toggle_grid)
        self.actionToggle_Active_Grid_Area.toggled.connect(MainWindow.action_toggle_active_grid_area)
        self.actionToggle_Selection.toggled.connect(MainWindow.action_toggle_select)
        self.actionToggle_Brush.toggled.connect(MainWindow.action_toggle_brush)
        self.actionToggle_Eraser.toggled.connect(MainWindow.action_toggle_eraser)
        self.actionRecenter.triggered.connect(MainWindow.action_center_clicked)
        self.actionToggle_LLM.toggled.connect(MainWindow.action_toggle_llm)
        self.actionExport_3.triggered.connect(MainWindow.action_export_image_triggered)
        self.actionMask_toggle.toggled.connect(MainWindow.action_toggle_mask_layer)
        self.actionUndo.triggered.connect(MainWindow.action_undo_triggered)
        self.actionRedo.triggered.connect(MainWindow.action_redo_triggered)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"AI Runner", None))
        self.actionNew.setText(QCoreApplication.translate("MainWindow", u"New", None))
#if QT_CONFIG(shortcut)
        self.actionNew.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+N", None))
#endif // QT_CONFIG(shortcut)
        self.actionImport.setText(QCoreApplication.translate("MainWindow", u"Import image", None))
#if QT_CONFIG(shortcut)
        self.actionImport.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+I", None))
#endif // QT_CONFIG(shortcut)
        self.actionExport.setText(QCoreApplication.translate("MainWindow", u"Export image", None))
#if QT_CONFIG(shortcut)
        self.actionExport.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+E", None))
#endif // QT_CONFIG(shortcut)
        self.actionCopy.setText(QCoreApplication.translate("MainWindow", u"Copy", None))
#if QT_CONFIG(shortcut)
        self.actionCopy.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+C", None))
#endif // QT_CONFIG(shortcut)
        self.actionPaste.setText(QCoreApplication.translate("MainWindow", u"Paste", None))
#if QT_CONFIG(shortcut)
        self.actionPaste.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+V", None))
#endif // QT_CONFIG(shortcut)
        self.actionUndo.setText(QCoreApplication.translate("MainWindow", u"Undo", None))
#if QT_CONFIG(tooltip)
        self.actionUndo.setToolTip(QCoreApplication.translate("MainWindow", u"Undo", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        self.actionUndo.setWhatsThis("")
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        self.actionUndo.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Z", None))
#endif // QT_CONFIG(shortcut)
        self.actionRedo.setText(QCoreApplication.translate("MainWindow", u"Redo", None))
#if QT_CONFIG(shortcut)
        self.actionRedo.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Y", None))
#endif // QT_CONFIG(shortcut)
        self.actionGrid.setText(QCoreApplication.translate("MainWindow", u"Grid", None))
#if QT_CONFIG(tooltip)
        self.actionGrid.setToolTip(QCoreApplication.translate("MainWindow", u"Grid settings", None))
#endif // QT_CONFIG(tooltip)
        self.actionPreferences.setText(QCoreApplication.translate("MainWindow", u"Path Preferences", None))
#if QT_CONFIG(tooltip)
        self.actionPreferences.setToolTip(QCoreApplication.translate("MainWindow", u"Modify model path and more", None))
#endif // QT_CONFIG(tooltip)
        self.actionCanvas_color.setText(QCoreApplication.translate("MainWindow", u"Canvas color", None))
#if QT_CONFIG(tooltip)
        self.actionCanvas_color.setToolTip(QCoreApplication.translate("MainWindow", u"Change the color of the canvas", None))
#endif // QT_CONFIG(tooltip)
        self.actionReset_Settings.setText(QCoreApplication.translate("MainWindow", u"Reset Settings to Default", None))
#if QT_CONFIG(tooltip)
        self.actionReset_Settings.setToolTip(QCoreApplication.translate("MainWindow", u"Reset all settings to default", None))
#endif // QT_CONFIG(tooltip)
        self.actionResize_on_Paste.setText(QCoreApplication.translate("MainWindow", u"Resize on Import", None))
#if QT_CONFIG(tooltip)
        self.actionResize_on_Paste.setToolTip(QCoreApplication.translate("MainWindow", u"Resize the image to active grid area when importing", None))
#endif // QT_CONFIG(tooltip)
        self.actionMemory.setText(QCoreApplication.translate("MainWindow", u"Memory", None))
        self.actionAbout.setText(QCoreApplication.translate("MainWindow", u"About", None))
        self.actionCheck_for_updates.setText(QCoreApplication.translate("MainWindow", u"Check for updates", None))
        self.actionOpen.setText(QCoreApplication.translate("MainWindow", u"Open", None))
        self.actionView.setText(QCoreApplication.translate("MainWindow", u"View", None))
        self.actionAdvanced.setText(QCoreApplication.translate("MainWindow", u"Memory Preferences", None))
#if QT_CONFIG(tooltip)
        self.actionAdvanced.setToolTip(QCoreApplication.translate("MainWindow", u"Change memory settings", None))
#endif // QT_CONFIG(tooltip)
        self.actionGaussian_Blur.setText(QCoreApplication.translate("MainWindow", u"Gaussian Blur", None))
        self.actionBox_Blur.setText(QCoreApplication.translate("MainWindow", u"Box Blur", None))
        self.actionUnsharp_Mask.setText(QCoreApplication.translate("MainWindow", u"Unsharp Mask", None))
        self.actionPixel_Art.setText(QCoreApplication.translate("MainWindow", u"Pixel Art", None))
        self.actionInvert.setText(QCoreApplication.translate("MainWindow", u"Invert", None))
#if QT_CONFIG(shortcut)
        self.actionInvert.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Shift+I", None))
#endif // QT_CONFIG(shortcut)
        self.actionSaturation.setText(QCoreApplication.translate("MainWindow", u"Saturation", None))
        self.actionColor_Balance.setText(QCoreApplication.translate("MainWindow", u"Color Balance", None))
        self.actionSave.setText(QCoreApplication.translate("MainWindow", u"Save", None))
#if QT_CONFIG(shortcut)
        self.actionSave.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+S", None))
#endif // QT_CONFIG(shortcut)
        self.actionLoad.setText(QCoreApplication.translate("MainWindow", u"Open", None))
#if QT_CONFIG(shortcut)
        self.actionLoad.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+O", None))
#endif // QT_CONFIG(shortcut)
        self.actionQuit.setText(QCoreApplication.translate("MainWindow", u"Quit", None))
#if QT_CONFIG(shortcut)
        self.actionQuit.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Q", None))
#endif // QT_CONFIG(shortcut)
        self.actionPixel_Art_2.setText(QCoreApplication.translate("MainWindow", u"Pixel Art", None))
        self.actionBug_report.setText(QCoreApplication.translate("MainWindow", u"Bug report", None))
        self.actionReport_vulnerability.setText(QCoreApplication.translate("MainWindow", u"Report vulnerability", None))
        self.actionDiscord.setText(QCoreApplication.translate("MainWindow", u"Discord", None))
        self.actionExtensions.setText(QCoreApplication.translate("MainWindow", u"Extensions", None))
#if QT_CONFIG(tooltip)
        self.actionExtensions.setToolTip(QCoreApplication.translate("MainWindow", u"Install, update and delete extensions", None))
#endif // QT_CONFIG(tooltip)
        self.actionShow_image_preview.setText(QCoreApplication.translate("MainWindow", u"Show image preview", None))
        self.actionImage_to_new_layer.setText(QCoreApplication.translate("MainWindow", u"Image to new layer", None))
#if QT_CONFIG(tooltip)
        self.actionImage_to_new_layer.setToolTip(QCoreApplication.translate("MainWindow", u"Send generated images to new layer", None))
#endif // QT_CONFIG(tooltip)
        self.actionAuto_export_images.setText(QCoreApplication.translate("MainWindow", u"Auto export images", None))
#if QT_CONFIG(tooltip)
        self.actionAuto_export_images.setToolTip(QCoreApplication.translate("MainWindow", u"Automatically export newly generated images to a folder", None))
#endif // QT_CONFIG(tooltip)
        self.actionImage_export_settings.setText(QCoreApplication.translate("MainWindow", u"Import / export preferences", None))
        self.actionCheck_for_latest_version_on_startup.setText(QCoreApplication.translate("MainWindow", u"Check for latest version on startup", None))
        self.actionModel_Merger.setText(QCoreApplication.translate("MainWindow", u"Model Merger", None))
        self.actionSaturation_Filter.setText(QCoreApplication.translate("MainWindow", u"Saturation", None))
        self.actionColor_Balance_2.setText(QCoreApplication.translate("MainWindow", u"Color Balance", None))
        self.actionRGB_Noise.setText(QCoreApplication.translate("MainWindow", u"RGB Noise", None))
        self.actionGaussian_Blur_2.setText(QCoreApplication.translate("MainWindow", u"Gaussian Blur", None))
        self.actionBox_Blur_2.setText(QCoreApplication.translate("MainWindow", u"Box Blur", None))
        self.actionRotate_90_clockwise.setText(QCoreApplication.translate("MainWindow", u"Rotate 90\u00b0 clockwise", None))
#if QT_CONFIG(shortcut)
        self.actionRotate_90_clockwise.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+R", None))
#endif // QT_CONFIG(shortcut)
        self.actionRotate_90_counter_clockwise.setText(QCoreApplication.translate("MainWindow", u"Rotate 90\u00b0 counter clockwise", None))
#if QT_CONFIG(shortcut)
        self.actionRotate_90_counter_clockwise.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Shift+R", None))
#endif // QT_CONFIG(shortcut)
        self.actionSave_prompt.setText(QCoreApplication.translate("MainWindow", u"Save prompt", None))
        self.actionShow_Active_Image_Area.setText(QCoreApplication.translate("MainWindow", u"Show Active Image Area", None))
        self.actionImage_Interpolation.setText(QCoreApplication.translate("MainWindow", u"Image Interpolation", None))
        self.actionImage_interpolation.setText(QCoreApplication.translate("MainWindow", u"Image Interpolation", None))
        self.actionFilm.setText(QCoreApplication.translate("MainWindow", u"Film", None))
#if QT_CONFIG(shortcut)
        self.actionFilm.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Shift+F", None))
#endif // QT_CONFIG(shortcut)
        self.actionDeterministic_generation.setText(QCoreApplication.translate("MainWindow", u"Deterministic generator", None))
        self.actionDark_mode.setText(QCoreApplication.translate("MainWindow", u"Dark mode", None))
        self.actionConsole_window.setText(QCoreApplication.translate("MainWindow", u"Console window", None))
        self.actionPrompt_Browser.setText(QCoreApplication.translate("MainWindow", u"Prompt browser", None))
        self.actionClear_all_prompts.setText(QCoreApplication.translate("MainWindow", u"Clear all prompts", None))
#if QT_CONFIG(tooltip)
        self.actionClear_all_prompts.setToolTip(QCoreApplication.translate("MainWindow", u"Remove text from all prompts and negative prompts", None))
#endif // QT_CONFIG(tooltip)
        self.actionUndo_2.setText(QCoreApplication.translate("MainWindow", u"Undo", None))
#if QT_CONFIG(shortcut)
        self.actionUndo_2.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Z", None))
#endif // QT_CONFIG(shortcut)
        self.actionRedo_2.setText(QCoreApplication.translate("MainWindow", u"Redo", None))
#if QT_CONFIG(shortcut)
        self.actionRedo_2.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Y", None))
#endif // QT_CONFIG(shortcut)
        self.actionCut.setText(QCoreApplication.translate("MainWindow", u"Cut", None))
#if QT_CONFIG(shortcut)
        self.actionCut.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+X", None))
#endif // QT_CONFIG(shortcut)
        self.actionHalftone.setText(QCoreApplication.translate("MainWindow", u"Halftone", None))
        self.actionQuick_Export.setText(QCoreApplication.translate("MainWindow", u"Fast Export", None))
#if QT_CONFIG(shortcut)
        self.actionQuick_Export.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Shift+E", None))
#endif // QT_CONFIG(shortcut)
        self.actionRegistration_error.setText(QCoreApplication.translate("MainWindow", u"Registration error", None))
        self.actionHuggingface_Cache_manager.setText(QCoreApplication.translate("MainWindow", u"Browse Huggingface Cache", None))
        self.actionModel_Manager.setText(QCoreApplication.translate("MainWindow", u"Model Manager", None))
        self.actionModel_Manager_2.setText(QCoreApplication.translate("MainWindow", u"Model Manager", None))
        self.actionControlNet.setText(QCoreApplication.translate("MainWindow", u"ControlNet", None))
        self.actionPrompt_Builder.setText(QCoreApplication.translate("MainWindow", u"Prompt Builder", None))
        self.actionEmbeddings.setText(QCoreApplication.translate("MainWindow", u"Embeddings", None))
        self.actionLoRA.setText(QCoreApplication.translate("MainWindow", u"LoRA", None))
        self.actionPen.setText(QCoreApplication.translate("MainWindow", u"Pen", None))
        self.actionStableDiffusion.setText(QCoreApplication.translate("MainWindow", u"Stable Diffusion", None))
        self.actionKandinsky.setText(QCoreApplication.translate("MainWindow", u"Kandinsky", None))
        self.actionShap_E.setText(QCoreApplication.translate("MainWindow", u"Shap-E", None))
        self.actionRun_setup_wizard.setText(QCoreApplication.translate("MainWindow", u"Run setup wizard", None))
        self.actionBrowse_AI_Runner_Path.setText(QCoreApplication.translate("MainWindow", u"Browse AI Runner Path", None))
        self.actionBrowse_Images_Path.setText(QCoreApplication.translate("MainWindow", u"Browse Images Path", None))
        self.actionBrowse_Images_Path_2.setText(QCoreApplication.translate("MainWindow", u"Browse Images", None))
        self.actionBrowse_Gifs_Path.setText(QCoreApplication.translate("MainWindow", u"Browse GIFs", None))
        self.actionTxt2img_Img2img_Models.setText(QCoreApplication.translate("MainWindow", u"Txt2img / Img2img", None))
        self.actionInpaint_Outpaint_Models.setText(QCoreApplication.translate("MainWindow", u"Inpaint / Outpaint", None))
        self.actionActive_Grid.setText(QCoreApplication.translate("MainWindow", u"Active Grid", None))
        self.actionDeterministic_Batches.setText(QCoreApplication.translate("MainWindow", u"Deterministic Batches", None))
        self.actionStandard_Batches.setText(QCoreApplication.translate("MainWindow", u"Standard Batches", None))
        self.actionLLM_beta.setText(QCoreApplication.translate("MainWindow", u"LLM (beta)", None))
        self.actionReset_Settings_2.setText(QCoreApplication.translate("MainWindow", u"Reset Settings", None))
        self.actionImage_Browser.setText(QCoreApplication.translate("MainWindow", u"Image Browser", None))
        self.actionModel_Manager_3.setText(QCoreApplication.translate("MainWindow", u"Model Manager", None))
        self.actionImport_input_image.setText(QCoreApplication.translate("MainWindow", u"Import input image", None))
        self.actionExport_input_image.setText(QCoreApplication.translate("MainWindow", u"Export input image", None))
        self.actionImport_2.setText(QCoreApplication.translate("MainWindow", u"Import image", None))
        self.actionExport_image.setText(QCoreApplication.translate("MainWindow", u"Export image", None))
        self.actionImport_Image.setText(QCoreApplication.translate("MainWindow", u"Import Image", None))
        self.actionExport_Image.setText(QCoreApplication.translate("MainWindow", u"Export Image", None))
        self.actionImport_3.setText(QCoreApplication.translate("MainWindow", u"Import", None))
        self.actionExport_2.setText(QCoreApplication.translate("MainWindow", u"Export", None))
        self.controlnet_import.setText(QCoreApplication.translate("MainWindow", u"Import Image", None))
        self.controlnet_export.setText(QCoreApplication.translate("MainWindow", u"Export Image", None))
        self.drawingpad_import.setText(QCoreApplication.translate("MainWindow", u"Import Image", None))
        self.drawingpad_export.setText(QCoreApplication.translate("MainWindow", u"Export Image", None))
        self.actionGenerate_while_drawing.setText(QCoreApplication.translate("MainWindow", u"Generate while drawing", None))
        self.actionOverlay_input_image.setText(QCoreApplication.translate("MainWindow", u"Overlay input image", None))
        self.enable_controlnet.setText(QCoreApplication.translate("MainWindow", u"Enabled", None))
        self.actionUnload_LLm.setText(QCoreApplication.translate("MainWindow", u"Unload LLM", None))
        self.actionUnload_Image_model.setText(QCoreApplication.translate("MainWindow", u"Unload Image model", None))
        self.actionClear_Memory.setText(QCoreApplication.translate("MainWindow", u"Clear Memory", None))
        self.actionSafety_Checker.setText(QCoreApplication.translate("MainWindow", u"Safety Checker", None))
        self.outpaint_import.setText(QCoreApplication.translate("MainWindow", u"Import", None))
        self.outpaint_export.setText(QCoreApplication.translate("MainWindow", u"Export", None))
        self.outpaint_enabled.setText(QCoreApplication.translate("MainWindow", u"Enabled", None))
        self.actionModel_Manager_4.setText(QCoreApplication.translate("MainWindow", u"Model Manager", None))
        self.actionRun_setup_wizard_2.setText(QCoreApplication.translate("MainWindow", u"Run Setup Wizard", None))
        self.actionStats.setText(QCoreApplication.translate("MainWindow", u"Stats", None))
        self.actionSettings.setText(QCoreApplication.translate("MainWindow", u"Preferences", None))
        self.actionToggle_LLM.setText(QCoreApplication.translate("MainWindow", u"Toggle LLM", None))
        self.actionToggle_Text_to_Speech.setText(QCoreApplication.translate("MainWindow", u"Toggle Text to Speech", None))
        self.actionToggle_Speech_to_Text.setText(QCoreApplication.translate("MainWindow", u"Toggle Speech to Text", None))
        self.actionToggle_Stable_Diffusion.setText(QCoreApplication.translate("MainWindow", u"Toggle Stable Diffusion", None))
        self.actionToggle_Controlnet.setText(QCoreApplication.translate("MainWindow", u"Toggle Controlnet", None))
        self.actionExport_3.setText(QCoreApplication.translate("MainWindow", u"Export Image", None))
        self.actionToggle_Active_Grid_Area.setText(QCoreApplication.translate("MainWindow", u"Toggle Active Grid Area", None))
        self.actionToggle_Selection.setText(QCoreApplication.translate("MainWindow", u"Toggle Selection", None))
        self.actionToggle_Brush.setText(QCoreApplication.translate("MainWindow", u"Toggle Brush", None))
        self.actionToggle_Grid.setText(QCoreApplication.translate("MainWindow", u"Toggle Grid", None))
        self.actionRecenter.setText(QCoreApplication.translate("MainWindow", u"Recenter", None))
        self.actionToggle_Eraser.setText(QCoreApplication.translate("MainWindow", u"Toggle Eraser", None))
        self.actionMask_toggle.setText(QCoreApplication.translate("MainWindow", u"Mask", None))
#if QT_CONFIG(tooltip)
        self.actionMask_toggle.setToolTip(QCoreApplication.translate("MainWindow", u"Toggle Mask", None))
#endif // QT_CONFIG(tooltip)
        self.menuFile.setTitle(QCoreApplication.translate("MainWindow", u"File", None))
        self.menuEdit.setTitle(QCoreApplication.translate("MainWindow", u"Edit", None))
        self.menuFilters.setTitle(QCoreApplication.translate("MainWindow", u"Filters", None))
        self.menuAbout.setTitle(QCoreApplication.translate("MainWindow", u"Help", None))
        self.menuImage.setTitle(QCoreApplication.translate("MainWindow", u"Canvas", None))
        self.menuView.setTitle(QCoreApplication.translate("MainWindow", u"View", None))
        self.menuTools.setTitle(QCoreApplication.translate("MainWindow", u"Tools", None))
        self.toolBar.setWindowTitle(QCoreApplication.translate("MainWindow", u"toolBar", None))
#if QT_CONFIG(tooltip)
        self.toolBar.setToolTip(QCoreApplication.translate("MainWindow", u"Export Image", None))
#endif // QT_CONFIG(tooltip)
        self.model_toolbar.setWindowTitle(QCoreApplication.translate("MainWindow", u"toolBar_2", None))
    # retranslateUi




def parse_template(template: dict) -> str:
    system_instructions = template["system_instructions"]
    model = template["model"]
    llm_category = template["llm_category"]
    template = template["template"]
    parsed_template = ""
    if llm_category == "causallm":
        if model in [
            "w4ffl35/Mistral-7B-Instruct-v0.3-4bit"
        ]:
            parsed_template = "\n".join((
                "[INST]<<SYS>>",
                system_instructions,
                "<</SYS>>",
                template,
                "[/INST]"
            ))
    return parsed_template



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'nullscream.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QFrame, QGridLayout, QGroupBox,
    QLabel, QLineEdit, QSizePolicy, QSplitter,
    QTextEdit, QWidget)

class Ui_nullscream(object):
    def setupUi(self, nullscream):
        if not nullscream.objectName():
            nullscream.setObjectName(u"nullscream")
        nullscream.resize(459, 644)
        self.gridLayout_3 = QGridLayout(nullscream)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.label = QLabel(nullscream)
        self.label.setObjectName(u"label")
        font = QFont()
        font.setBold(True)
        self.label.setFont(font)

        self.gridLayout_3.addWidget(self.label, 0, 0, 1, 1)

        self.line = QFrame(nullscream)
        self.line.setObjectName(u"line")
        self.line.setFrameShape(QFrame.Shape.HLine)
        self.line.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout_3.addWidget(self.line, 1, 0, 1, 1)

        self.splitter = QSplitter(nullscream)
        self.splitter.setObjectName(u"splitter")
        self.splitter.setOrientation(Qt.Orientation.Vertical)
        self.groupBox = QGroupBox(self.splitter)
        self.groupBox.setObjectName(u"groupBox")
        self.gridLayout = QGridLayout(self.groupBox)
        self.gridLayout.setObjectName(u"gridLayout")
        self.blocked = QTextEdit(self.groupBox)
        self.blocked.setObjectName(u"blocked")

        self.gridLayout.addWidget(self.blocked, 1, 0, 1, 1)

        self.lineEdit_2 = QLineEdit(self.groupBox)
        self.lineEdit_2.setObjectName(u"lineEdit_2")

        self.gridLayout.addWidget(self.lineEdit_2, 0, 0, 1, 1)

        self.splitter.addWidget(self.groupBox)
        self.groupBox_2 = QGroupBox(self.splitter)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.gridLayout_2 = QGridLayout(self.groupBox_2)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.allowed = QTextEdit(self.groupBox_2)
        self.allowed.setObjectName(u"allowed")

        self.gridLayout_2.addWidget(self.allowed, 1, 0, 1, 1)

        self.lineEdit = QLineEdit(self.groupBox_2)
        self.lineEdit.setObjectName(u"lineEdit")

        self.gridLayout_2.addWidget(self.lineEdit, 0, 0, 1, 1)

        self.splitter.addWidget(self.groupBox_2)

        self.gridLayout_3.addWidget(self.splitter, 2, 0, 1, 1)


        self.retranslateUi(nullscream)
        self.lineEdit_2.textEdited.connect(nullscream.filter_blocked)
        self.lineEdit.textChanged.connect(nullscream.filter_allowed)

        QMetaObject.connectSlotsByName(nullscream)
    # setupUi

    def retranslateUi(self, nullscream):
        nullscream.setWindowTitle(QCoreApplication.translate("nullscream", u"Form", None))
        self.label.setText(QCoreApplication.translate("nullscream", u"NULLSCREAM", None))
        self.groupBox.setTitle(QCoreApplication.translate("nullscream", u"Blocked", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("nullscream", u"Allowed", None))
    # retranslateUi




import os

from airunner.data.models.settings_models import AIModels
from airunner.enums import SignalCode
from airunner.windows.main.pipeline_mixin import PipelineMixin
from airunner.workers.worker import Worker


class ModelScannerWorker(
    Worker,
    PipelineMixin
):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        PipelineMixin.__init__(self)

    def handle_message(self):
        self.scan_for_models()
        self.remove_missing_models()

    def scan_for_models(self):
        self.logger.debug("Scan for models")
        # look at model path and determine if we can import existing local models
        # first look at all files and folders inside of the model paths
        diffusers_folders = ["scheduler", "text_encoder", "tokenizer", "unet", "vae"]
        models = []
        model_path = os.path.expanduser(
            os.path.join(
                self.path_settings.base_path, "art/models",
            )
        )
        if not os.path.exists(model_path):
            self.logger.error(f"Model path does not exist: {model_path}")
            return
        # find all folders inside of model_path, each of those folders is a model version
        with os.scandir(model_path) as dir_object:
            # check if dir_object is a directory
            self.logger.debug(f"Scan for models {model_path}")
            for version_entry in dir_object:
                version = version_entry.name
                path = os.path.join(model_path, version)
                self.logger.debug(f"Scan directory {path}")
                with os.scandir(path) as action_object:
                    for action_item in action_object:
                        action = action_item.name
                        if "controlnet_processors" in action_item.path:
                            continue
                        with os.scandir(action_item.path) as file_object:
                            for file_item in file_object:
                                model = AIModels()
                                model.name = os.path.basename(file_item.path)
                                model.path = file_item.path
                                model.branch = "main"
                                model.version = version
                                model.category = "stablediffusion"
                                model.pipeline_action = action
                                model.enabled = True
                                model.model_type = "art"
                                model.is_default = False
                                if file_item.is_file():  # ckpt or safetensors file
                                    if file_item.name.endswith(".ckpt") or file_item.name.endswith(".safetensors"):
                                        name = file_item.name.replace(".ckpt", "").replace(".safetensors", "")
                                        model.name = name
                                    else:
                                        model = None
                                elif file_item.is_dir():  # diffusers folder
                                    is_diffusers_directory = True
                                    for diffuser_folder in diffusers_folders:
                                        if not os.path.exists(os.path.join(file_item.path, diffuser_folder)):
                                            is_diffusers_directory = False
                                            model = None
                                    if is_diffusers_directory:
                                        model.name = file_item.name

                                if model:
                                    models.append(model)
        self.emit_signal(SignalCode.AI_MODELS_SAVE_OR_UPDATE_SIGNAL, {"models": models})

    def remove_missing_models(self):
        self.logger.debug("Remove missing models")
        # remove all models that are not in the model path
        model_path = os.path.expanduser(
            os.path.join(
                self.path_settings.base_path, "art/models",
            )
        )
        if not os.path.exists(model_path):
            self.logger.error(f"Model path does not exist: {model_path}")
            return
        existing_models = self.session.query(AIModels).all()
        for model in existing_models:
            if not os.path.exists(model.path):
                self.session.delete(model)
                self.session.commit()
                self.logger.debug(f"Removed missing model: {model.name}")


import inspect
import queue
import threading

from PySide6.QtCore import Signal, QThread, QObject

from airunner.enums import QueueType, SignalCode, WorkerState
from airunner.mediator_mixin import MediatorMixin
from airunner.settings import SLEEP_TIME_IN_MS
from airunner.windows.main.settings_mixin import SettingsMixin


class Worker(QObject, MediatorMixin, SettingsMixin):
    queue_type = QueueType.GET_NEXT_ITEM
    finished = Signal()
    prefix = "Worker"

    def __init__(self, signals=None):
        self.signals = signals or []
        MediatorMixin.__init__(self)
        
        super().__init__()
        self.state = WorkerState.HALTED
        self.running = False
        self.queue = queue.Queue()
        self.items = {}
        self.current_index = 0
        self.paused = False
        self.register(SignalCode.QUIT_APPLICATION, self.stop)
        self.register_signals()

        threading.Thread(target=self.start_worker_thread).start()

    def start_worker_thread(self):
        pass

    def register_signals(self):
        for signal in self.signals:
            self.register(signal[0], signal[1])

    def start(self):
        import traceback
        traceback.format_exc()
        self.run()

    def run(self):
        if self.queue_type == QueueType.NONE:
            return
        self.logger.debug("Starting worker")
        self.running = True
        while self.running:
            self.preprocess()
            try:
                msg = self.get_item_from_queue()
                if msg is not None:
                    if len(inspect.signature(self.handle_message).parameters) == 0:
                        self.handle_message()
                    else:
                        self.handle_message(msg)
            except queue.Empty:
                msg = None
            if self.paused:
                self.logger.debug("Paused")
                while self.paused:
                    QThread.msleep(SLEEP_TIME_IN_MS)
                self.logger.debug("Resumed")
            QThread.msleep(SLEEP_TIME_IN_MS)

    def preprocess(self):
        pass
    
    def get_item_from_queue(self):
        if self.queue_type == QueueType.GET_LAST_ITEM:
            msg = self.get_last_item()
        else:
            msg = self.get_next_item()
        return msg
    
    def get_last_item(self):
        msg = None
        index = None
        while not self.queue.empty():
            try:
                index = self.queue.get_nowait()
                if index in self.items:
                    break
            except queue.Empty:
                pass
        if index in self.items:
            msg = self.items.pop(index)
            self.items = {}
            self.queue.empty()
            return msg
        else:
            return None

    def get_next_item(self):
        try:
            index = self.queue.get_nowait()
            msg = self.items.pop(index, None)
            return msg
        except queue.Empty:
            return None

    def pause(self):
        self.state = WorkerState.PAUSED

    def unpause(self):
        if self.state == WorkerState.PAUSED:
            self.state = WorkerState.RUNNING

    def resume(self):
        self.paused = False

    def handle_message(self, message):
        raise NotImplementedError

    def add_to_queue(self, message):
        if (
            type(message) is dict and
            "options" in message and
            message["options"]["empty_queue"]
        ):
            self.empty_queue()

        if self.queue_type == QueueType.GET_LAST_ITEM:
            self.items = {}
            self.queue.empty()
            self.current_index = 0

        self.items[self.current_index] = message
        self.queue.put(self.current_index)
        self.current_index += 1

    def empty_queue(self):
        self.queue = queue.Queue()
        self.items = {}
        self.current_index = 0

    def stop(self):
        self.logger.debug("Stopping")
        self.running = False
        self.finished.emit()

    def cancel(self):
        self.logger.debug("Canceling")
        while not self.queue.empty():
            self.queue.get()



from airunner.handlers.base_handler import BaseHandler
from airunner.enums import ModelType


class TTSHandler(BaseHandler):
    """
    Generates speech from given text. 
    Responsible for managing the model, processor, vocoder, and speaker embeddings.
    Generates using either the SpeechT5 model.

    Use from a worker to avoid blocking the main thread.
    """
    target_model = None
    model_class_ = None
    processor_class_ = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.model_type = ModelType.TTS
        self.model_class = "tts"
        self._engine = None

    def interrupt_process_signal(self):
        pass
    
    def offload_to_cpu(self):
        pass
    
    def move_to_device(self, device=None):
        pass

    def load(self, target_model=None):
        pass

    def unload(self):
        pass

    def run(self):
        pass

    def initialize(self):
        pass

    def do_generate(self, message):
        pass




from airunner.data.models.settings_models import Message
from airunner.enums import SignalCode
from airunner.widgets.base_widget import BaseWidget
from airunner.widgets.llm.templates.message_ui import Ui_message

from PySide6.QtGui import QFontDatabase, QFont
from PySide6.QtWidgets import QTextEdit, QApplication, QWidget
from PySide6.QtGui import QFontMetrics
from PySide6.QtCore import Qt, QSize, Slot, QEvent
from PySide6.QtCore import Signal


class AutoResizingTextEdit(QTextEdit):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.document().contentsChanged.connect(self.sizeChange)


class MessageWidget(BaseWidget):
    widget_class_ = Ui_message
    textChanged = Signal()

    def __init__(self, *args, **kwargs):
        self.name = kwargs.pop("name")
        self.message = kwargs.pop("message")
        self.conversation_id = kwargs.pop("conversation_id")
        self.message_id = kwargs.pop("message_id")
        if self.message_id is None:
            session = self.session
            message = session.query(Message).filter(Message.conversation_id == self.conversation_id).order_by(
                Message.id.desc()).first()
            if message is not None:
                self.message_id = message.id
        self.is_bot = kwargs.pop("is_bot")
        super().__init__(*args, **kwargs)
        self.ui.content.setReadOnly(True)
        self.ui.content.insertPlainText(self.message)
        self.ui.content.document().contentsChanged.connect(self.sizeChange)
        self.ui.user_name.setText(f"{self.name}")
        self.register(SignalCode.APPLICATION_SETTINGS_CHANGED_SIGNAL, self.on_application_settings_changed_signal)
        self.font_family = None
        self.font_size = None
        self.set_chat_font()

        if self.is_bot:
            self.ui.message_container.setProperty("class", "alternate")

        self.ui.copy_button.setVisible(False)
        self.ui.delete_button.setVisible(False)
        self.ui.message_container.installEventFilter(self)
        self.set_cursor(Qt.CursorShape.ArrowCursor)

    def set_cursor(self, cursor_type):
        self.ui.message_container.setCursor(cursor_type)
        for child in self.ui.message_container.findChildren(QWidget):
            child.setCursor(cursor_type)

    def eventFilter(self, obj, event):
        if obj == self.ui.message_container:
            if event.type() == QEvent.Type.Enter:
                self.ui.copy_button.setVisible(True)
                self.ui.delete_button.setVisible(True)
            elif event.type() == QEvent.Type.Leave:
                self.ui.copy_button.setVisible(False)
                self.ui.delete_button.setVisible(False)
        return super().eventFilter(obj, event)

    def on_application_settings_changed_signal(self):
        self.set_chat_font()

    def set_chat_font(self):
        font_setting = self.get_font_setting_by_name("chat")
        font_family = font_setting.font_family
        font_size = font_setting.font_size
        if self.font_family != font_family or self.font_size != font_size:
            self.font_family = font_family
            self.font_size = font_size
            # Check if the font family is available
            available_families = QFontDatabase().families()
            if self.font_family in available_families:
                font = QFont(self.font_family, self.font_size)
            else:
                font = QFont("Ubuntu", self.font_size)  # Fallback to Ubuntu if Arial is not available
            font.setFamilies([
                font.family(),
                "Noto Color Emoji",
            ])
            self.ui.content.setFont(font)

    def set_content_size(self):
        doc_height = self.ui.content.document().size().height()
        doc_width = self.ui.content.document().size().width()
        self.setMinimumHeight(int(doc_height) + 45)
        self.setMinimumWidth(int(doc_width))

    def sizeChange(self):
        self.set_content_size()
        self.textChanged.emit()

    def resizeEvent(self, event):
        self.set_content_size()
        super().resizeEvent(event)

    def sizeHint(self):
        fm = QFontMetrics(self.font())
        h = fm.height() * (self.ui.content.document().lineCount() + 1)
        return QSize(self.width(), h)

    def minimumSizeHint(self):
        return self.sizeHint()

    def update_message(self, text):
        self.message += text

        # strip double spaces from self.message
        self.message = self.message.replace("  ", " ")

        self.ui.content.setPlainText(self.message)

    @Slot()
    def delete(self):
        session = self.session
        message = session.query(Message).filter(Message.id == self.message_id).first()
        session.delete(message)
        session.commit()
        self.setParent(None)
        self.deleteLater()

    @Slot()
    def copy(self):
        clipboard = QApplication.clipboard()
        clipboard.setText(self.message)



# src/airunner/alembic/env.py
import os

from logging.config import fileConfig
from sqlalchemy import engine_from_config, pool, MetaData
from alembic import context

config = context.config
db_path = os.path.expanduser("~/.local/share/airunner/data/airunner.db")
config.set_main_option("sqlalchemy.url", f"sqlite:///{db_path}")

# check if db file exists
if not os.path.exists(db_path):
    print(f"Database file not found at {db_path}")

# Import your models here
from airunner.data.models.settings_models import (
    Conversation, Message, Summary,
    ApplicationSettings, ActiveGridSettings, ControlnetSettings,
    ImageToImageSettings, OutpaintSettings, DrawingPadSettings, MetadataSettings,
    GeneratorSettings, LLMGeneratorSettings, TTSSettings,
    SpeechT5Settings, EspeakSettings, STTSettings, Schedulers, BrushSettings,
    GridSettings, PathSettings, MemorySettings, Chatbot, TargetFiles, TargetDirectories,
    AIModels, ShortcutKeys, Lora, SavedPrompt, Embedding, PromptTemplate, ControlnetModel,
    FontSetting, PipelineModel, WindowSettings
)

# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
fileConfig(config.config_file_name)

# Create a single MetaData object
metadata = MetaData()

# Bind all models to the single MetaData object
for model in [
    ApplicationSettings, ActiveGridSettings, ControlnetSettings,
    ImageToImageSettings, OutpaintSettings, DrawingPadSettings, MetadataSettings,
    GeneratorSettings, LLMGeneratorSettings, TTSSettings,
    SpeechT5Settings, EspeakSettings, STTSettings, Schedulers, BrushSettings,
    GridSettings, PathSettings, MemorySettings, Chatbot, TargetFiles, TargetDirectories,
    AIModels, ShortcutKeys, Lora, SavedPrompt, Embedding, PromptTemplate, ControlnetModel,
    FontSetting, PipelineModel, WindowSettings, Conversation, Message, Summary
]:
    model.metadata = metadata

# Combine all model's MetaData objects here
target_metadata = metadata

# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.

def run_migrations_offline():
    """Run migrations in 'offline' mode.
    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well. By skipping the Engine creation
    we don't even need a DBAPI to be available.
    Calls to context.execute() here emit the given string to the
    script output.
    """
    url = config.get_main_option("sqlalchemy.url")
    context.configure(
        url=url,
        target_metadata=target_metadata,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
    )

    with context.begin_transaction():
        context.run_migrations()

def run_migrations_online():
    """Run migrations in 'online' mode.
    In this scenario we need to create an Engine
    and associate a connection with the context.
    """
    connectable = engine_from_config(
        config.get_section(config.config_ini_section),
        prefix="sqlalchemy.",
        poolclass=pool.NullPool,
    )

    with connectable.connect() as connection:
        context.configure(
            connection=connection, target_metadata=target_metadata
        )

        with context.begin_transaction():
            context.run_migrations()

if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()



from PIL import Image
from airunner.filters.base_filter import BaseFilter


class ColorBalanceFilter(BaseFilter):
    def apply_filter(self, image, _do_reset):
        image = image.convert("RGBA")
        red, green, blue, alpha = image.split()
        red = red.point(lambda i: i + (i * self.cyan_red))
        green = green.point(lambda i: i + (i * self.magenta_green))
        blue = blue.point(lambda i: i + (i * self.yellow_blue))
        image = Image.merge("RGBA", (red, green, blue, alpha))
        return image



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'export_preferences.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox, QFrame,
    QGridLayout, QLabel, QLineEdit, QPushButton,
    QSizePolicy, QSpacerItem, QWidget)

class Ui_export_preferences(object):
    def setupUi(self, export_preferences):
        if not export_preferences.objectName():
            export_preferences.setObjectName(u"export_preferences")
        export_preferences.resize(705, 602)
        self.gridLayout_2 = QGridLayout(export_preferences)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.actionAuto_export_images = QCheckBox(export_preferences)
        self.actionAuto_export_images.setObjectName(u"actionAuto_export_images")
        font = QFont()
        font.setBold(True)
        self.actionAuto_export_images.setFont(font)

        self.gridLayout_2.addWidget(self.actionAuto_export_images, 0, 0, 1, 1)

        self.line = QFrame(export_preferences)
        self.line.setObjectName(u"line")
        self.line.setFrameShape(QFrame.Shape.HLine)
        self.line.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout_2.addWidget(self.line, 1, 0, 1, 2)

        self.label = QLabel(export_preferences)
        self.label.setObjectName(u"label")
        self.label.setFont(font)

        self.gridLayout_2.addWidget(self.label, 2, 0, 1, 1)

        self.label_2 = QLabel(export_preferences)
        self.label_2.setObjectName(u"label_2")
        font1 = QFont()
        font1.setPointSize(10)
        self.label_2.setFont(font1)

        self.gridLayout_2.addWidget(self.label_2, 3, 0, 1, 1)

        self.image_path = QLineEdit(export_preferences)
        self.image_path.setObjectName(u"image_path")

        self.gridLayout_2.addWidget(self.image_path, 4, 0, 1, 1)

        self.image_path_browse_button = QPushButton(export_preferences)
        self.image_path_browse_button.setObjectName(u"image_path_browse_button")

        self.gridLayout_2.addWidget(self.image_path_browse_button, 4, 1, 1, 1)

        self.line_3 = QFrame(export_preferences)
        self.line_3.setObjectName(u"line_3")
        self.line_3.setFrameShape(QFrame.Shape.HLine)
        self.line_3.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout_2.addWidget(self.line_3, 5, 0, 1, 2)

        self.label_5 = QLabel(export_preferences)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setFont(font)

        self.gridLayout_2.addWidget(self.label_5, 6, 0, 1, 1)

        self.line_2 = QFrame(export_preferences)
        self.line_2.setObjectName(u"line_2")
        self.line_2.setFrameShape(QFrame.Shape.HLine)
        self.line_2.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout_2.addWidget(self.line_2, 8, 0, 1, 2)

        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.metadata_iterations = QCheckBox(export_preferences)
        self.metadata_iterations.setObjectName(u"metadata_iterations")

        self.gridLayout.addWidget(self.metadata_iterations, 7, 0, 1, 1)

        self.metadata_model_branch = QCheckBox(export_preferences)
        self.metadata_model_branch.setObjectName(u"metadata_model_branch")

        self.gridLayout.addWidget(self.metadata_model_branch, 4, 1, 1, 1)

        self.metadata_seed = QCheckBox(export_preferences)
        self.metadata_seed.setObjectName(u"metadata_seed")

        self.gridLayout.addWidget(self.metadata_seed, 5, 0, 1, 1)

        self.metadata_model = QCheckBox(export_preferences)
        self.metadata_model.setObjectName(u"metadata_model")

        self.gridLayout.addWidget(self.metadata_model, 3, 1, 1, 1)

        self.label_3 = QLabel(export_preferences)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setFont(font)

        self.gridLayout.addWidget(self.label_3, 0, 0, 1, 1)

        self.metadata_scheduler = QCheckBox(export_preferences)
        self.metadata_scheduler.setObjectName(u"metadata_scheduler")

        self.gridLayout.addWidget(self.metadata_scheduler, 5, 1, 1, 1)

        self.export_metadata = QCheckBox(export_preferences)
        self.export_metadata.setObjectName(u"export_metadata")

        self.gridLayout.addWidget(self.export_metadata, 0, 1, 1, 1)

        self.metadata_samples = QCheckBox(export_preferences)
        self.metadata_samples.setObjectName(u"metadata_samples")

        self.gridLayout.addWidget(self.metadata_samples, 2, 1, 1, 1)

        self.verticalSpacer = QSpacerItem(17, 37, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout.addItem(self.verticalSpacer, 8, 0, 1, 1)

        self.metadata_scale = QCheckBox(export_preferences)
        self.metadata_scale.setObjectName(u"metadata_scale")

        self.gridLayout.addWidget(self.metadata_scale, 4, 0, 1, 1)

        self.metadata_negative_prompt = QCheckBox(export_preferences)
        self.metadata_negative_prompt.setObjectName(u"metadata_negative_prompt")

        self.gridLayout.addWidget(self.metadata_negative_prompt, 3, 0, 1, 1)

        self.metadata_ddim_eta = QCheckBox(export_preferences)
        self.metadata_ddim_eta.setObjectName(u"metadata_ddim_eta")

        self.gridLayout.addWidget(self.metadata_ddim_eta, 6, 1, 1, 1)

        self.metadata_prompt = QCheckBox(export_preferences)
        self.metadata_prompt.setObjectName(u"metadata_prompt")

        self.gridLayout.addWidget(self.metadata_prompt, 2, 0, 1, 1)

        self.metadata_steps = QCheckBox(export_preferences)
        self.metadata_steps.setObjectName(u"metadata_steps")

        self.gridLayout.addWidget(self.metadata_steps, 6, 0, 1, 1)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.gridLayout.addItem(self.horizontalSpacer, 0, 2, 1, 1)

        self.metadata_clip_skip = QCheckBox(export_preferences)
        self.metadata_clip_skip.setObjectName(u"metadata_clip_skip")

        self.gridLayout.addWidget(self.metadata_clip_skip, 2, 2, 1, 1)

        self.metadata_version = QCheckBox(export_preferences)
        self.metadata_version.setObjectName(u"metadata_version")

        self.gridLayout.addWidget(self.metadata_version, 3, 2, 1, 1)

        self.metadata_lora = QCheckBox(export_preferences)
        self.metadata_lora.setObjectName(u"metadata_lora")

        self.gridLayout.addWidget(self.metadata_lora, 4, 2, 1, 1)

        self.metadata_embeddings = QCheckBox(export_preferences)
        self.metadata_embeddings.setObjectName(u"metadata_embeddings")

        self.gridLayout.addWidget(self.metadata_embeddings, 5, 2, 1, 1)

        self.metadata_timestamp = QCheckBox(export_preferences)
        self.metadata_timestamp.setObjectName(u"metadata_timestamp")

        self.gridLayout.addWidget(self.metadata_timestamp, 6, 2, 1, 1)

        self.metadata_strength = QCheckBox(export_preferences)
        self.metadata_strength.setObjectName(u"metadata_strength")

        self.gridLayout.addWidget(self.metadata_strength, 7, 1, 1, 1)

        self.metadata_controlnet = QCheckBox(export_preferences)
        self.metadata_controlnet.setObjectName(u"metadata_controlnet")

        self.gridLayout.addWidget(self.metadata_controlnet, 7, 2, 1, 1)

        self.label_4 = QLabel(export_preferences)
        self.label_4.setObjectName(u"label_4")
        font2 = QFont()
        font2.setPointSize(10)
        font2.setBold(True)
        self.label_4.setFont(font2)

        self.gridLayout.addWidget(self.label_4, 1, 0, 1, 3)


        self.gridLayout_2.addLayout(self.gridLayout, 9, 0, 1, 2)

        self.image_type_dropdown = QComboBox(export_preferences)
        self.image_type_dropdown.setObjectName(u"image_type_dropdown")

        self.gridLayout_2.addWidget(self.image_type_dropdown, 7, 0, 1, 2)


        self.retranslateUi(export_preferences)
        self.actionAuto_export_images.toggled.connect(export_preferences.action_toggle_automatically_export_images)
        self.image_path.textEdited.connect(export_preferences.image_export_path_text_edited)
        self.image_path_browse_button.clicked.connect(export_preferences.action_clicked_button_browse)
        self.image_type_dropdown.currentTextChanged.connect(export_preferences.action_image_type_text_changed)
        self.export_metadata.toggled.connect(export_preferences.action_toggled_export_metadata)
        self.metadata_prompt.toggled.connect(export_preferences.action_toggled_prompt)
        self.metadata_negative_prompt.toggled.connect(export_preferences.action_toggled_negative_prompt)
        self.metadata_scale.toggled.connect(export_preferences.action_toggled_scale)
        self.metadata_seed.toggled.connect(export_preferences.action_toggled_seed)
        self.metadata_steps.toggled.connect(export_preferences.action_toggled_steps)
        self.metadata_iterations.toggled.connect(export_preferences.action_toggled_iterations)
        self.metadata_samples.toggled.connect(export_preferences.action_toggled_samples)
        self.metadata_model.toggled.connect(export_preferences.action_toggled_model)
        self.metadata_model_branch.toggled.connect(export_preferences.action_toggled_model_branch)
        self.metadata_scheduler.toggled.connect(export_preferences.action_toggled_scheduler)
        self.metadata_ddim_eta.toggled.connect(export_preferences.action_toggled_ddim)
        self.metadata_strength.toggled.connect(export_preferences.action_toggled_strength)
        self.metadata_clip_skip.toggled.connect(export_preferences.action_toggled_clip_skip)
        self.metadata_version.toggled.connect(export_preferences.action_toggled_version)
        self.metadata_lora.toggled.connect(export_preferences.action_toggled_lora)
        self.metadata_embeddings.toggled.connect(export_preferences.action_toggled_embeddings)
        self.metadata_timestamp.toggled.connect(export_preferences.action_toggled_timestamp)
        self.metadata_controlnet.toggled.connect(export_preferences.action_toggled_controlnet)

        QMetaObject.connectSlotsByName(export_preferences)
    # setupUi

    def retranslateUi(self, export_preferences):
        export_preferences.setWindowTitle(QCoreApplication.translate("export_preferences", u"Form", None))
        self.actionAuto_export_images.setText(QCoreApplication.translate("export_preferences", u"Automatically export images", None))
        self.label.setText(QCoreApplication.translate("export_preferences", u"Image export folder", None))
        self.label_2.setText(QCoreApplication.translate("export_preferences", u"Folder to auto export images to", None))
        self.image_path_browse_button.setText(QCoreApplication.translate("export_preferences", u"Browse", None))
        self.label_5.setText(QCoreApplication.translate("export_preferences", u"Image type", None))
        self.metadata_iterations.setText(QCoreApplication.translate("export_preferences", u"Iterations", None))
        self.metadata_model_branch.setText(QCoreApplication.translate("export_preferences", u"Model Branch", None))
        self.metadata_seed.setText(QCoreApplication.translate("export_preferences", u"Seed", None))
        self.metadata_model.setText(QCoreApplication.translate("export_preferences", u"Model", None))
        self.label_3.setText(QCoreApplication.translate("export_preferences", u"Metadata", None))
        self.metadata_scheduler.setText(QCoreApplication.translate("export_preferences", u"Scheduler", None))
#if QT_CONFIG(tooltip)
        self.export_metadata.setToolTip(QCoreApplication.translate("export_preferences", u"<html><head/><body><p>Export metadata along with images. </p><p>Only works with PNG files.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.export_metadata.setText(QCoreApplication.translate("export_preferences", u"Export metadata", None))
        self.metadata_samples.setText(QCoreApplication.translate("export_preferences", u"Samples", None))
        self.metadata_scale.setText(QCoreApplication.translate("export_preferences", u"Scale", None))
        self.metadata_negative_prompt.setText(QCoreApplication.translate("export_preferences", u"Negative prompt", None))
        self.metadata_ddim_eta.setText(QCoreApplication.translate("export_preferences", u"DDIM ETA", None))
        self.metadata_prompt.setText(QCoreApplication.translate("export_preferences", u"Prompt", None))
        self.metadata_steps.setText(QCoreApplication.translate("export_preferences", u"Steps", None))
        self.metadata_clip_skip.setText(QCoreApplication.translate("export_preferences", u"Clip Skip", None))
        self.metadata_version.setText(QCoreApplication.translate("export_preferences", u"Version", None))
        self.metadata_lora.setText(QCoreApplication.translate("export_preferences", u"LoRA", None))
        self.metadata_embeddings.setText(QCoreApplication.translate("export_preferences", u"Embeddings", None))
        self.metadata_timestamp.setText(QCoreApplication.translate("export_preferences", u"Timestamp", None))
        self.metadata_strength.setText(QCoreApplication.translate("export_preferences", u"Strength", None))
        self.metadata_controlnet.setText(QCoreApplication.translate("export_preferences", u"Controlnet", None))
        self.label_4.setText(QCoreApplication.translate("export_preferences", u"Choose which metadata to include with exported images", None))
    # retranslateUi




################################################################
# Importing this module sets the Hugging Face environment
# variables for the application.
################################################################
import os.path
import sys
import signal
import traceback
from functools import partial
from pathlib import Path

from PySide6 import QtCore
from PySide6.QtCore import (
    QObject,
    QTimer
)
from PySide6.QtGui import (
    QGuiApplication,
    QPixmap,
    Qt,
    QWindow
)
from PySide6.QtWidgets import (
    QApplication,
    QSplashScreen
)
from sqlalchemy import distinct

from airunner.app_installer import AppInstaller
from airunner.enums import SignalCode
from airunner.mediator_mixin import MediatorMixin
from airunner.windows.main.settings_mixin import SettingsMixin
from airunner.data.models.settings_models import ApplicationSettings, AIModels
from airunner.windows.main.main_window import MainWindow


class App(
    QObject,
    SettingsMixin,
    MediatorMixin
):
    """
    The main application class for AI Runner.
    This class can be run as a GUI application or as a socket server.
    """
    def __init__(
        self,
        main_window_class: QWindow = None,
        defendatron=None
    ):
        """
        Initialize the application and run as a GUI application or a socket server.
        :param main_window_class: The main window class to use for the application.
        """
        self.main_window_class_ = main_window_class or MainWindow
        self.app = None
        self.defendatron = defendatron
        self.splash = None

        """
        Mediator and Settings mixins are initialized here, enabling the application
        to easily access the application settings dictionary.
        """
        MediatorMixin.__init__(self)
        super(App, self).__init__()

        self.register(SignalCode.LOG_LOGGED_SIGNAL, self.on_log_logged_signal)

        self.start()
        self.run_setup_wizard()
        self.run()

    def run_setup_wizard(self):
        application_settings = self.session.query(ApplicationSettings).first()
        if application_settings.run_setup_wizard:
            AppInstaller()

    def on_log_logged_signal(self, data: dict):
        message = data["message"].split(" - ")
        self.update_splash_message(self.splash, message[4])

    def start(self):
        """
        Conditionally initialize and display the setup wizard.
        :return:
        """
        signal.signal(signal.SIGINT, self.signal_handler)
        QApplication.setAttribute(Qt.ApplicationAttribute.AA_UseDesktopOpenGL)
        self.app = QApplication([])

    def run(self):
        """
        Run as a GUI application.
        A splash screen is displayed while the application is loading
        and a main window is displayed once the application is ready.

        Override this method to run the application in a different mode.
        """

        # Continue with application execution
        self.splash = self.display_splash_screen(self.app)

        # Show the main application window
        QTimer.singleShot(
            50,
            partial(
                self.show_main_application,
                self.app,
                self.splash
            )
        )
        sys.exit(self.app.exec())

    @staticmethod
    def signal_handler(
        _signal,
        _frame
    ):
        """
        Handle the SIGINT signal in a clean way.
        :param _signal:
        :param _frame:
        :return: None
        """
        print("\nExiting...")
        try:
            app = QApplication.instance()
            app.quit()
            sys.exit(0)
        except Exception as e:
            print(e)
            sys.exit(0)

    @staticmethod
    def display_splash_screen(app):
        """
        Display a splash screen while the application is loading.
        :param app:
        :return:
        """
        screens = QGuiApplication.screens()
        try:
            screen = screens.at(0)
        except AttributeError:
            screen = screens[0]

        base_dir = Path(os.path.dirname(os.path.realpath(__file__)))
        stylesheet_path = base_dir / "images" / "splashscreen.png"
        pixmap = QPixmap(stylesheet_path)
        splash = QSplashScreen(
            screen,
            pixmap,
            QtCore.Qt.WindowType.WindowStaysOnTopHint
        )
        splash.show()
        App.update_splash_message(splash, f"Loading AI Runner")
        app.processEvents()
        return splash

    @staticmethod
    def update_splash_message(splash, message: str):
        splash.showMessage(
            message,
            QtCore.Qt.AlignmentFlag.AlignBottom | QtCore.Qt.AlignmentFlag.AlignCenter,
            QtCore.Qt.GlobalColor.white
        )

    def show_main_application(
        self,
        app,
        splash
    ):
        """
        Show the main application window.
        :param app:
        :param splash:
        :return:
        """
        try:
            window = self.main_window_class_(
                defendatron=self.defendatron
            )
        except Exception as e:
            traceback.print_exc()
            print(e)
            splash.finish(None)
            sys.exit("""
                An error occurred while initializing the application.
                Please report this issue on GitHub or Discord."
            """)
        app.main_window = window
        splash.finish(window)
        window.raise_()



from PySide6.QtCore import Qt
from PySide6.QtGui import (
    QColor,
    QPainter,
    QPen,
    QBrush,
    QPixmap,
    QCursor
)


def CircleCursor(outline_color, fill_color, pixmap_size=32):
    pixmap_size = int(pixmap_size)
    # create a pixmap with the desired size for the cursor shape
    pixmap = QPixmap(pixmap_size, pixmap_size)
    pixmap.fill(Qt.GlobalColor.transparent)  # make the background of the pixmap transparent

    # draw a circle in the pixmap
    painter = QPainter(pixmap)
    painter.setRenderHint(QPainter.RenderHint.Antialiasing)  # make the edges of the circle smoother
    pen = QPen(QColor(outline_color))
    pen.setWidth(2)
    brush = QBrush(QColor(fill_color))
    painter.setPen(pen)
    painter.setBrush(brush)
    painter.drawEllipse(1, 1, pixmap_size - 2, pixmap_size - 2)

    # draw an inner circle in the pixmap
    pen = QPen(QColor(Qt.GlobalColor.black))
    pen.setWidth(2)
    brush = QBrush(QColor(fill_color))
    painter.setPen(pen)
    painter.setBrush(brush)
    painter.drawEllipse(2, 2, pixmap_size - 4, pixmap_size - 4)

    painter.end()

    # create a cursor from the pixmap
    return QCursor(pixmap, pixmap_size // 2, pixmap_size // 2)
